/* 
   $Id: schedule.c,v 1.5 2001/12/27 08:21:15 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgsched.

   sgsched is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgsched is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgsched; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

#include "schedule.h"
#include "calendar.h"

static struct sched_data *sched_top  = NULL;
static struct sched_data *sched_tail = NULL;

static struct sched_data *load_schedule(char *fn);
static void               save_schedule(char *fn, struct sched_data *list);

static void insert_data(struct sched_data *dest, struct sched_data *src);
static void remove_data(struct sched_data *list);

static void insert_data(struct sched_data *dest, struct sched_data *src)
{
  if(!dest || !src) return;

  src->prev  = dest;
  dest->next = src;
}

static void remove_data(struct sched_data *list)
{
  struct sched_data *next, *prev;

  if(list == NULL) return;

  prev = list->prev;
  next = list->next;

  if(prev) prev->next = next;
  if(next) next->prev = prev;

  if(list->data) free(list->data);
  free(list);
}

void sched_add_data(struct sched_data *data)
{
  if(sched_tail){
    insert_data(sched_tail, data);
    sched_tail = sched_tail->next;  
  }
}

void sched_del_data(struct sched_data *data)
{
  remove_data(data);

  sched_tail = sched_top;
  while(sched_tail->next){
    sched_tail = sched_tail->next;
  }
}

static struct sched_data *
load_schedule(char *fn)
{
  FILE              *fp;
  int               i;
  char              *s;
  char              str[2048];
  int               c;
  struct sched_data *tmp = NULL, *top, *tail;
#ifdef OLD_FORMAT
  long              l;
  int               size;
#endif

  top  = (struct sched_data *) calloc(1, sizeof(struct sched_data));

  fp = fopen(fn, "r"); 
  if(fp == NULL){
    fprintf(stderr, "I can't open  schedule book file:%s.\n",fn);
    return top;
  }
#ifdef OLD_FORMAT
  fseek(fp, 0, SEEK_END);
  size = ftell(fp);
  if(size == 0)    return top;
  fseek(fp, 0, SEEK_SET);

  i = 0;

  tail = top;

  while(size > i){
    tmp  = (struct sched_data *) calloc(1, sizeof(struct sched_data));
    i += fread(&tmp->time,  sizeof(time_t), 1, fp) * sizeof(time_t);
    i += fread(&tmp->start, sizeof(time_t), 1, fp) * sizeof(time_t);
    i += fread(&tmp->end,   sizeof(time_t), 1, fp) * sizeof(time_t);
    i += fread(&tmp->flag,  sizeof(long),   1, fp) * sizeof(time_t);

    i += fread(&l, sizeof(long), 1, fp) * sizeof(long);
    if(l){
      s = (char *) calloc(1, sizeof(char) * (l + 1));
      i += fread(s, sizeof(char), l, fp) * sizeof(char);
      tmp->data = s;
    }

    insert_data(tail, tmp);
    tail = tail->next;
  }

#else
  tail = top;
  s = str;
  i = 0;
  while((c = fgetc(fp)) != EOF){
    switch(c){
    case '(': 
      i = 0;
      s = str;
      break;

    case ')':
      *s = '\0';
      if(i == 6){
	if(strlen(str) > 0) tmp->data = strdup(str);
      }

      insert_data(tail, tmp);
      tail = tail->next;
      i = 0;
      break;

    case '\n':
      break;

    case '\"':
      while( (c = fgetc(fp)) != EOF){
	if(c == '\"') break;
	*s++ = c;
      }
      break;

    case ' ':
      *s = '\0';

      if(s != str){
	switch(i){
	case 0:
	  tmp  = (struct sched_data *) calloc(1, sizeof(struct sched_data));
	  if(strlen(str) > 0) tmp->time = atoi(str);
	  break;
	case 1:
	  if(strlen(str) > 0) tmp->start = atoi(str);
	  break;
	case 2:
	  if(strlen(str) > 0) tmp->end = atoi(str);
	  break;
	case 3:
	  if(strlen(str) > 0) tmp->flag = atoi(str);
	  break;
	case 4:
	  if(strlen(str) > 0) tmp->location = strdup(str);
	  break;
	case 5:
	  if(strlen(str) > 0) tmp->person = strdup(str);
	  break;
	case 6:
	  if(strlen(str) > 0) tmp->data = strdup(str);
	  break;
	}
      }
      i++;
      s = str;
      break;
      
    default:
      *s++ = c;
      break;
    }
  }
#endif
  
  fclose(fp);
  return top;
}

void save_schedule(char *fn, struct sched_data *list)
{
  struct sched_data *tmp;
  FILE              *fp;
#ifdef OLD_FORMAT
  long              l;
#endif
  
  if(!list) return;

  fp = fopen(fn, "w"); 
  if(fp == NULL){
    fprintf(stderr, "I can't open  schedule book file:%s.\n",fn);
    return;
  }

  tmp = list->next;
#ifdef OLD_FORMAT
  while(tmp){
    fwrite(&tmp->time,  sizeof(time_t), 1, fp);
    fwrite(&tmp->start, sizeof(time_t), 1, fp);
    fwrite(&tmp->end,   sizeof(time_t), 1, fp);
    fwrite(&tmp->flag,  sizeof(long),   1, fp);

    if(tmp->data){
      l = strlen(tmp->data);
      fwrite(&l, sizeof(long), 1, fp);
      fwrite(tmp->data, sizeof(char), l, fp);
    }else{
      l = 0;
      fwrite(&l, sizeof(long), 1, fp);
    }
    
    tmp = tmp->next;
  }
#else
  while(tmp){
    fprintf(fp, "(");
    fprintf(fp, "%ld", (long) tmp->time);
    fprintf(fp, " %ld",(long) tmp->start);
    fprintf(fp, " %ld",(long) tmp->end);
    fprintf(fp, " %ld",(long) tmp->flag);
    
    fprintf(fp,  " \"");
    if(tmp->location){
      fprintf(fp, "%s", tmp->location);
    }
    fprintf(fp,  "\"");

    fprintf(fp,  " \"");
    if(tmp->person){
      fprintf(fp, "%s", tmp->person);
    }
    fprintf(fp,  "\"");

    fprintf(fp,  " \"");
    if(tmp->data){
      fprintf(fp, "%s", tmp->data);
    }
    fprintf(fp,  "\"");
    fprintf(fp, ")\n");

    tmp = tmp->next;
  }
#endif
  fclose(fp);
}


/* save schedules to specified file */
void sched_save_list(char *fn)
{
  save_schedule(fn, sched_top);
}

/* load schedules from specified file */
struct sched_data *sched_load_list(char *fn)
{
  sched_top = load_schedule(fn);

  if(sched_top){
    sched_tail = sched_top;
    while(sched_tail->next){
      sched_tail = sched_tail->next;
    }
  }
  
  return sched_top;
}

struct sched_data *
make_next_weekly_schedule(struct sched_data *sc, int y, int m, int d)
{
  struct tm   *tm;
  struct sched_data *ret;
  int w;
  
  tm = localtime(&sc->start);
  if(tm == NULL) return NULL;
  w = get_dayweek(y, m, d);
  
  while(tm->tm_wday != w){ 
    next_day(y, m, d);

    w =  get_dayweek(y, m, d);
  }

  ret = (struct sched_data *) calloc(1, sizeof(struct sched_data));

  ret->start = get_time_t(y, m, d, tm->tm_hour, tm->tm_min, tm->tm_sec);
  ret->end   = ret->start + sc->end - sc->start;
  ret->orig  = sc;
  ret->next  = NULL;
  ret->prev  = NULL;
  ret->flag  = sc->flag;
  ret->data  = sc->data;

  return ret;
}

static int comp_time_by_day(time_t t1, time_t t2)
{
  struct tm   *tm;
  struct date a, b;
  
  tm = localtime(&t1);
  a.year = tm->tm_year + 1900;
  a.mon  = tm->tm_mon  + 1;
  a.mday = tm->tm_mday;

  tm = localtime(&t2);
  b.year = tm->tm_year + 1900;
  b.mon  = tm->tm_mon  + 1;
  b.mday = tm->tm_mday;

  return comp_date(&a, &b);
}

int is_pm(time_t t)
{
  struct tm *tm;

  tm = localtime(&t);

  if(tm){
    if(tm->tm_hour < 12)       return 0;
    else if(tm->tm_hour >= 12) return 1;
  }

  return 0;
}

static int is_same_mday(time_t t, int mday)
{
  struct tm *tm;

  tm = localtime(&t);

  if(tm){
    if(mday == tm->tm_mday) return 1;
    else                    return 0;
  }

  return 0;
}

/* create schedule list of specified range */
struct sched_data *
sched_create_list(int sy, int sm, int sd, int ey, int em, int ed)
{
  time_t             t1, t2, sc_start, sc_end, diff;
  struct sched_data  *top = NULL, *tail = NULL, *sc = NULL;
  struct sched_data  *tmp = NULL;
  struct tm          *tm;
  int                y, m, d;

  t1 = get_time_t(sy, sm, sd,  0,  0,  0);
  t2 = get_time_t(ey, em, ed, 23, 59, 59);
  
  if(!sched_top || !sched_top->next) return NULL;

  top  = (struct sched_data *) calloc(1, sizeof(struct sched_data));
  tail = top;

  tmp = sched_top->next;

  while(tmp){
    if(tmp->flag & NORMAL_SCHEDULE){
      if((t1 <= tmp->end && tmp->end <= t2) 
	 || (t1 <= tmp->start && tmp->start <= t2)
	 || (t1 <= tmp->start && tmp->end <= t2)
	 || (t1 >= tmp->start && tmp->end >= t2)){

	sc = (struct sched_data *) calloc(1, sizeof(struct sched_data));
	*sc = *tmp;
	sc->orig = tmp;
	sc->next = NULL;
	insert_data(tail, sc);
	tail = tail->next;
      }

    }else if(tmp->flag & WEEKLY_SCHEDULE){
      y = sy, m = sm, d = sd;
      diff = tmp->end - tmp->start;

      while(1){
	sc = make_next_weekly_schedule(tmp, y, m, d);

	if((t1 <= sc->start && sc->start <= t2)
	   || (t1 <= sc->end && sc->end <= t2)
	   || (t1 <= sc->start && sc->end <= t2)
	   || (t1 >= sc->start && sc->end >= t2)){

	  tm = localtime(&sc->start);
	  y = tm->tm_year + 1900, m = tm->tm_mon + 1, d = tm->tm_mday;

	  if(comp_time_by_day(tmp->start, sc->start) <= 0){
	    insert_data(tail, sc);
	    tail = tail->next;
	  }else{
	    free(sc);
	  }

	  next_day(y, m, d);
	}else{
	  free(sc);
	  break;
	}
      }

    }else if(tmp->flag & MONTHLY_SCHEDULE){
      tm = localtime(&tmp->start);

      sc_start = get_time_t(sy, sm, tm->tm_mday, 
			    tm->tm_hour, tm->tm_min, tm->tm_sec);

      diff = tmp->end - tmp->start;
      sc_end = sc_start + diff;

      if((t1 <= sc_start && sc_start <= t2)
	 || (t1 <= sc_end && sc_end <= t2)
	 || (t1 <= sc_start && sc_end <= t2)
	 || (t1 >= sc_start && sc_end >= t2)){
	
	if(comp_time_by_day(tmp->start, sc_start) <= 0){	
	  if(tm->tm_mday > get_day_of_month(sy,sm)) continue;

	  sc = (struct sched_data *) calloc(1, sizeof(struct sched_data));
	  *sc = *tmp;
	  sc->orig = tmp;
	  tm = localtime(&sc->start);
	  sc->start = get_time_t(sy, sm, tm->tm_mday,
				 tm->tm_hour, tm->tm_min, tm->tm_sec);
	  sc->end = sc->start + diff;
	  
	  sc->next = NULL;
	  insert_data(tail, sc);
	  tail = tail->next;
	}
      }
    }

    tmp = tmp->next;
  }

  return top;
}

void sched_destroy_list(struct sched_data *list)
{
  struct sched_data *t;

  while(list != NULL){
    t = list;
    list = list->next;
    free(t);
  }
}

/* get a schedule of specified date */
struct sched_data *
get_schedule_time(int year, int mon, int day, int hour, int min)
{
  struct sched_data *tmp, *ret;
  time_t            t;

  if(sched_top == NULL) return NULL;
  
  tmp = sched_top->next;

  t = get_time_t(year, mon, day, hour, min, 0);
  
  while(tmp){
    if(tmp->start <= t && t <= tmp->end){
      ret = (struct sched_data *) calloc(1, sizeof(struct sched_data));
      *ret = *tmp;
      ret->next = NULL;
      ret->prev = NULL;
      return ret;
    }

    tmp = tmp->next;
  }

  return NULL;
}

/* get the first schedule after specified date */
struct sched_data *
get_schedule_after_time(int year, int mon, int day, int hour, int min)
{
  struct sched_data *tmp;
  time_t            t;

  if(sched_top == NULL) return NULL;
  
  t = get_time_t(year, mon, day, hour, min, 0);

  tmp = sched_create_list(year, mon, day, year, mon, day);  
  tmp = tmp->next;

  while(tmp){
    if(t <= tmp->start && is_same_mday(tmp->start, day)){
      if(!(tmp->flag & DAYOFF_SCHEDULE))   return tmp;
    }else{
      tmp = tmp->next;
      if(tmp->prev) free(tmp->prev);
    }
  }

  return NULL;
}

/* append a schedule to specified file */
int append_schedule_data(char *fn, struct sched_data *sched)
{
  FILE *fp;  
#ifdef OLD_FORMAT
  int  l;
#endif

  fp = fopen(fn, "a"); 
  if(fp == NULL){
    fprintf(stderr, "I can't open  schedule book file:%s.\n",fn);
    return 0;
  }
  
#ifdef OLD_FORMAT
  fwrite(&sched->time,  sizeof(time_t), 1, fp);
  fwrite(&sched->start, sizeof(time_t), 1, fp);
  fwrite(&sched->end,   sizeof(time_t), 1, fp);
  fwrite(&sched->flag,  sizeof(long),   1, fp);
  
  if(sched->data){
    l = strlen(sched->data);
    fwrite(&l, sizeof(long), 1, fp);
    fwrite(sched->data, sizeof(char), l, fp);
  }else{
    l = 0;
    fwrite(&l, sizeof(long), 1, fp);
  }

#else
  fprintf(fp, "(");
  fprintf(fp, "%ld", (long) sched->time);
  fprintf(fp, " %ld",(long) sched->start);
  fprintf(fp, " %ld",(long) sched->end);
  fprintf(fp, " %ld",(long) sched->flag);
    
  fprintf(fp,  " \"");
  if(sched->location){
    fprintf(fp, "%s", sched->location);
  }
  fprintf(fp,  "\"");
  
  fprintf(fp,  " \"");
  if(sched->person){
    fprintf(fp, "%s", sched->person);
  }
  fprintf(fp,  "\"");
  
  fprintf(fp,  " \"");
  if(sched->data){
    fprintf(fp, "%s", sched->data);
  }
  fprintf(fp,  "\"");
  fprintf(fp, ")\n");
#endif

  fclose(fp);
  return 1;
}

static int number_of_schedule(time_t t1, time_t t2)
{
  time_t             sc_start, sc_end, diff;
  struct sched_data  *tmp = NULL, *sc = NULL;
  struct tm          *tm;
  int                y, m, d;
  int                sy, sm, sd;
  int                n;
  
  if(!sched_top) return 0;

  tmp = sched_top->next;

  tm = localtime(&t1);
  sy = tm->tm_year + 1900;
  sm = tm->tm_mon  + 1;
  sd = tm->tm_mday;

  n = 0;
  while(tmp){
    if(tmp->flag & NORMAL_SCHEDULE){
      if((t1 <= tmp->end && tmp->end < t2) 
	 || (t1 <= tmp->start && tmp->start < t2)
	 || (t1 <= tmp->start && tmp->end < t2)
	 || (t1 >= tmp->start && tmp->end > t2)){
	n++;
      }
    }else if(tmp->flag & WEEKLY_SCHEDULE){
      y = sy, m = sm, d = sd;
      diff = tmp->end - tmp->start;
      while(1){
	sc = make_next_weekly_schedule(tmp, y, m, d);
	
	if((t1 <= sc->start && sc->start <= t2)
	   ||(t1 <= sc->end  && sc->end <= t2)
	   ||(t1 <= sc->start && sc->end <= t2)
	   ||(t1 >= sc->start && sc->end >= t2)){

	  tm = localtime(&sc->start);
	  y = tm->tm_year + 1900, m = tm->tm_mon + 1, d = tm->tm_mday;

	  if(comp_time_by_day(tmp->start, sc->start) <= 0) n++;
	  
	  free(sc);

	  next_day(y, m, d);
	}else{
	  free(sc);
	  break;
	}
      }

    }else if(tmp->flag & MONTHLY_SCHEDULE){
      tm = localtime(&tmp->start);

      sc_start = get_time_t(sy, sm, tm->tm_mday, 
			    tm->tm_hour, tm->tm_min, tm->tm_sec);
      diff = tmp->end - tmp->start;
      sc_end = sc_start + diff;

      if((t1 <= sc_start && sc_start < t2)
	 || (t1 <= sc_end && sc_end < t2)
	 || (t1 <= sc_start && sc_end < t2)
	 || (t1 >= sc_start && sc_end > t2)){
	
	if(comp_time_by_day(tmp->start, sc_start) <= 0){	
	  if(tm->tm_mday > get_day_of_month(sy,sm)) continue;
	  else                                      n++;
	}
      }
    }

    tmp = tmp->next;
  }

  return n;
}

/* get the number of schedules of specified day */
int get_number_of_schedule(int sy, int sm, int sd)
{
  time_t      t1, t2;
  int         n;
  
  if(!sched_top) return 0;

  t1 = get_time_t(sy, sm, sd,  0,  0,  0);
  t2 = get_time_t(sy, sm, sd, 23, 59, 59);

  n = number_of_schedule(t1, t2);

  return n;
}

/* get the first schedule between specified day to days */
struct sched_data *
get_schedule_after_day(int sy, int sm, int sd, int days)
{
  struct sched_data  *tmp = NULL;
  struct sched_data  *sc = NULL;
  struct tm   *tm;
  time_t      t1;
  
  if(!sched_top) return NULL;

  t1 = get_time_t(sy, sm, sd, 0, 0, 0) + (days * 86400);
  tm = localtime(&t1);

  tmp = sched_create_list(sy, sm, sd, tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);

  tmp = sort_schedule_list(tmp);

  sc = (struct sched_data *) calloc(1, sizeof(struct sched_data));
  *sc = *tmp->next;
  sc->next = NULL;
  sc->prev = NULL;

  sched_destroy_list(tmp);

  return sc;
}

static int comp_sched_data(struct sched_data **a, struct sched_data **b)
{
  if((*a) == NULL && (*b) == NULL)   return  0;
  else if((*a) == NULL)              return -1;
  else if((*b) == NULL)              return  1;

  if((*a)->start > (*b)->start)      return  1;
  else if((*a)->start < (*b)->start) return -1;
  else{
    if((*a)->flag == HOLIDAY_SCHEDULE)       return -1;
    else if((*b)->flag == HOLIDAY_SCHEDULE)  return  1;
    else                                     return  0;
  }
}

struct sched_data *
sort_schedule_list(struct sched_data *list)
{
  struct sched_data **array;
  struct sched_data *tmp;
  struct sched_data *tail;

  int i, j;

  if(!list) return NULL;

  i = 0, tmp = list->next;
  while(tmp){
    i++;
    tmp = tmp->next;
  }
  
  array = (struct sched_data **) calloc(i, sizeof(struct sched_data *));
  i = 0, tmp = list->next;
  while(tmp){
    array[i++] = tmp;
    tmp = tmp->next;
  }

  qsort(array, i, sizeof(struct sched_data *), (void *)comp_sched_data);
  
  tail = list;
  for(j = 0; j < i; j++){
    array[j]->prev = tail;
    tail->next = array[j];
    tail = tail->next;
  }
  tail->next = NULL;

  free(array);
  return list;
}

static struct sched_data *is_schedule(time_t t)
{
  struct tm         *tm;
  struct sched_data *tmp = NULL;

  tm  = localtime(&t);
  tmp = get_schedule_time(tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
			  tm->tm_hour, tm->tm_min);

  return tmp;
}

/* search the free time after specified date */
struct sched_data *
search_free_time(int year, int mon, int mday, int wday,  int hour, int min, int interval, int max)
{
  struct sched_data *tmp = NULL;
  struct sched_data *ret;

  struct tm         *tm;
  time_t            t, t1, t2, s;

  time(&t);
  tm = localtime(&t);

  tm->tm_year = year - 1900;
  tm->tm_mon  = mon  - 1;
  tm->tm_mday = mday;

  tm->tm_hour = hour;
  tm->tm_min  = min;
  tm->tm_sec  = 0;
  s = mktime(tm);
  t = s;

  while(diff_time_t(t, s) < max){
    if(wday != -1){
      while(tm->tm_wday != wday){  
	t += 86400;
	tm = localtime(&t);
      }
    }
    
    t1 = t;
    t2 = t + (interval * 60);
    
    tmp = is_schedule(t1);
    
    if(tmp != NULL){
      free(tmp);
      t += 86400;
      continue;
    }
    
    tmp = is_schedule(t2);
    
    if(tmp == NULL){
      ret = (struct sched_data *) calloc(1, sizeof(struct sched_data));
      ret->start = t1;
      ret->end   = t2;
      return ret;
    }

    t += 86400;
  }

  return NULL;
}
