/* 
   $Id: sgsched_cal.c,v 1.6 2001/12/27 08:21:16 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgsched.

   sgsched is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgsched is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgsched; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "config.h"

#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sgsched.h"

#include "leftarrow.xpm"
#include "rightarrow.xpm"
#include "arrow_gl_r.xpm"
#include "arrow_gr_r.xpm"

#define  START_CALX 16
#define  START_CALY 32

#define  ARROW_WIDTH  16
#define  ARROW_HEIGHT 14

static GtkWidget *calendar;
static GdkPixmap *pixmap;
static GdkFont   *font;

static GdkPixmap *left;
static GdkBitmap *left_mask;

static GdkPixmap *right;
static GdkBitmap *right_mask;

static GdkPixmap *right_green;
static GdkBitmap *right_green_mask;
static GdkPixmap *right_glay;
static GdkBitmap *right_glay_mask;

GdkColor glay    = {0, 0xcfff, 0xcfff, 0xcfff}; 
GdkColor l_glay  = {0, 0xdfff, 0xdfff, 0xdfff}; 
GdkColor d_glay  = {0, 0x7fff, 0x7fff, 0x7fff}; 
GdkColor black   = {0,      0,      0,      0}; 
GdkColor white   = {0, 0xffff, 0xffff, 0xffff}; 

GdkColor red     = {0, 0xffff,      0,      0}; 
GdkColor d_red   = {0, 0x7fff,      0,      0}; 
GdkColor blue    = {0,      0,      0, 0xffff}; 
GdkColor d_blue  = {0,      0,      0, 0x7fff}; 
GdkColor green   = {0,      0, 0xcfff,      0}; 
GdkColor d_green = {0,      0, 0x7fff,      0}; 
GdkColor cyan    = {0,      0, 0xffff, 0xffff}; 

char *month_name[12] = {
  N_("1"), N_("2"), N_("3"), N_("4"),  N_("5"),  N_("6"),
  N_("7"), N_("8"), N_("9"), N_("10"), N_("11"), N_("12") 
};
  
char *weekday_name[7] = {
  N_("Sun"), N_("Mon"), N_("Tue"), N_("Wed"),  
  N_("Thu"), N_("Fri"), N_("Sat") 
};


int  get_start_dayweek(int year, int month);
void draw_days(int pos, int day, GdkColor *color, int am, int pm);
void draw_left_arrow(int x, int y);
void draw_right_arrow(int x, int y);
void draw_right_green_arrow(int x, int y);
void draw_right_glay_arrow(int x, int y);
void draw_button(int x, int y, int w, int h, int state, char *str);
void draw_month_button(int x, int y, int w, int h, int state, char *str);
void draw_week_button(int x, int y, int w, int h, int state, char *str);

void set_date_by_pos(int x, int y);
void next_press(int i);
void prev_press(int i);
void button_press(GtkWidget *widget, GdkEventButton *event);
void button_release(GtkWidget *widget, GdkEventButton *event);
int  configure_event(GtkWidget *widget, GdkEventConfigure *event);
int  expose_event(GtkWidget *widget, GdkEventExpose *event);

extern int    c_state[32];
extern int    current_year;
extern int    current_month;
extern int    current_day;
extern int    current_week;
extern struct sched_data *sc_current;
extern struct sched_data *sc_previous;

void init_holiday()
{
  DIR        *dirp;
  char       *home;
  char       fn[BUFSIZ];
  char       dname[BUFSIZ];

  if ((home = getenv("HOME")) != NULL) {
    sprintf(dname, "%s/sgsched",home);
  } else {
    sprintf(dname, "sgsched");
  }

  dirp = opendir(dname);

  if(!dirp) {
    mkdir(dname, 0700);
  } else {
    closedir(dirp);
  }

  sprintf(fn,"%s/holiday", RULEDIR);
  load_holiday(fn);

  if ((home = getenv("HOME")) != NULL) {
    sprintf(fn, "%s/sgsched/memorial", home);
  } else {
    sprintf(fn,"%s/memorial", RULEDIR);
  }
  load_memorial(fn);

}

int get_start_dayweek(int year, int month)
{
  int weekday;

  weekday = get_dayweek(year, month, 1);

  return weekday;
}

void draw_days(int pos, int day, GdkColor *color, int am, int pm)
{
  char s[256];
  int  h, w, a, x, y, gx, gy;

  GdkGC *gc;

  w = (calendar->allocation.width - START_CALX)/ 7;
  a = font->ascent;
  h = (calendar->allocation.height - START_CALY)/6;

  gc = gdk_gc_new(calendar->window);

  sprintf(s, "%d", day);
  gx = (w - gdk_string_width(font, s)) / 2 ;
  gy = (h - (font->ascent + font->descent))/2 + a;

  x = (pos % 7) * w + START_CALX;
  y = (pos / 7) * h + START_CALY; 

  gdk_gc_set_foreground (gc, color);
  gdk_draw_string(pixmap, font, gc, x + gx, y + gy, s);

  gdk_gc_set_line_attributes(gc, 2,
			     GDK_LINE_SOLID,
			     GDK_CAP_NOT_LAST,
			     GDK_JOIN_MITER);

  if(am){
    gdk_gc_set_foreground (gc, &red);
    gdk_draw_line(pixmap, gc, x + 1, y + 1, x + w - 1, y + 1);
    gdk_draw_line(pixmap, gc, x + 1, y + 1, x + 1, y + h - 1);
  }

  if(pm){
    gdk_gc_set_foreground (gc, &blue);
    gdk_draw_line(pixmap, gc, x + 1, y + h - 1, x + w - 1, y + h - 1);
    gdk_draw_line(pixmap, gc, x + w - 1, y + 1, x + w - 1, y + h - 1);
  }

  gdk_gc_destroy(gc);
}

void draw_left_arrow(int x, int y)
{
  GdkGC *gc;

  if(pixmap && left){
    gc = gdk_gc_new(calendar->window);
    gdk_gc_set_clip_mask(gc, left_mask);
    gdk_gc_set_clip_origin(gc, x, y);
    gdk_draw_pixmap(pixmap, gc, left, 0, 0, x, y, ARROW_WIDTH, ARROW_HEIGHT);
    gdk_gc_destroy(gc);
  }
}

void draw_right_arrow(int x, int y)
{
  GdkGC *gc;

  if(pixmap && right){
    gc = gdk_gc_new(calendar->window);
    gdk_gc_set_clip_mask(gc, right_mask);
    gdk_gc_set_clip_origin(gc, x, y);
    gdk_draw_pixmap(pixmap, gc, right, 0, 0, x, y, ARROW_WIDTH, ARROW_HEIGHT);
    gdk_gc_destroy(gc);
  }
}

void draw_right_green_arrow(int x, int y)
{
  GdkGC *gc;

  if(pixmap && right){
    gc = gdk_gc_new(calendar->window);
    gdk_gc_set_clip_mask(gc, right_green_mask);
    gdk_gc_set_clip_origin(gc, x, y);
    gdk_draw_pixmap(pixmap, gc, right_green, 0, 0, x, y, ARROW_WIDTH, ARROW_HEIGHT);
    gdk_gc_destroy(gc);
  }
}

void draw_right_glay_arrow(int x, int y)
{
  GdkGC *gc;

  if(pixmap && right){
    gc = gdk_gc_new(calendar->window);
    gdk_gc_set_clip_mask(gc, right_glay_mask);
    gdk_gc_set_clip_origin(gc, x, y);
    gdk_draw_pixmap(pixmap, gc, right_glay, 0, 0, x, y, ARROW_WIDTH, ARROW_HEIGHT);
    gdk_gc_destroy(gc);
  }
}

void draw_month_button(int x, int y, int w, int h, int state, char *str)
{
  GdkGC *gc;
  int a = 0, gx, gy;

  gc = gdk_gc_new(calendar->window);

  if(state == 1) gdk_gc_set_foreground (gc, &black);
  else           gdk_gc_set_foreground (gc, &white);
  gdk_draw_line(pixmap, gc, x, y, x, y + h);
  gdk_draw_line(pixmap, gc, x, y, x + w, y);

  if(state == 1) gdk_gc_set_foreground (gc, &white);
  else           gdk_gc_set_foreground (gc, &black);
  gdk_draw_line(pixmap, gc, x + w, y, x + w, y + h);
  gdk_draw_line(pixmap, gc, x, y + h, x + w, y + h);
  
  if(state == 1) gdk_gc_set_foreground (gc, &green);
  else           gdk_gc_set_foreground (gc, &glay);
  gdk_draw_rectangle(pixmap, gc, TRUE, x + 1, y + 1, w - 1, h - 1);

  a = font->ascent;
  gx = (w - gdk_string_width(calendar->style->font, str))/2;
  gy = (h - (font->ascent + font->descent))/2 + a;

#if 0
  if(state == 1) gdk_gc_set_foreground (gc, &black);
  else           gdk_gc_set_foreground (gc, &white);
  gdk_draw_string(pixmap, font, gc, x + gx + 1, y + gy + 1, str);
#endif

  if(state == 1) gdk_gc_set_foreground (gc, &white);
  else           gdk_gc_set_foreground (gc, &black);
  gdk_draw_string(pixmap, font, gc, x + gx, y + gy, str);

  gdk_gc_destroy(gc);
}

void draw_week_button(int x, int y, int w, int h, int state, char *str)
{
  int gx, gy;
  GdkGC *gc;

  gc = gdk_gc_new(calendar->window);
  gdk_gc_set_foreground (gc, &glay);
  gdk_draw_rectangle(pixmap, gc, TRUE, x, y, w, h);

  gx = (w - ARROW_WIDTH)/2;
  gy = (h - ARROW_HEIGHT)/2;

  if(state == 1) draw_right_green_arrow(x + gx, y+ gy);
  else           draw_right_glay_arrow(x + gx, y+ gy);

  gdk_gc_destroy(gc);
}

void draw_calendar(int year, int month, int day)
{
  int  i, j;
  int  wday, mday, pday, nday;
  int  am, pm;
    
  char s[256];
  GdkGC *gc;

  int h, w, a, g, m;

  if(month <= 0 || month > 12)                            return;
  if(day   <= 0 || day   > get_day_of_month(year, month)) return;

  w = (calendar->allocation.width - START_CALX)/ 7;

  a = font->ascent;
  h = (calendar->allocation.height - START_CALY)/ 6;

  m = calendar->allocation.width - ((calendar->allocation.width - START_CALX)% 7);

  gc = calendar->style->white_gc;
  gdk_draw_rectangle(pixmap, gc, TRUE,
		     0, 0, m, calendar->allocation.height);

  gc = gdk_gc_new(calendar->window);
  gdk_gc_set_foreground (gc, &glay);
  gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, m, ARROW_WIDTH);
  gdk_gc_destroy(gc);

  /* draw year and month */

  draw_left_arrow(0, 0);
  draw_right_arrow(ARROW_WIDTH + 48, 0);

  sprintf(s, "%04d", year);
  g = (48 - gdk_string_width(calendar->style->font, s))/2 + ARROW_WIDTH;
#if 0
  gc = calendar->style->white_gc;
  gdk_draw_string(pixmap, font, gc, g + 1, a + 1, s);
#endif
  gc = calendar->style->black_gc;
  gdk_draw_string(pixmap, font, gc, g, a, s);

  draw_left_arrow(m - ((ARROW_WIDTH * 2) + 48), 0);
  draw_right_arrow(m - ARROW_WIDTH, 0);

  sprintf(s, "%s", _(month_name[(month - 1)]));
  draw_month_button(m - (ARROW_WIDTH + 48) , 0, 46, 14,
		    get_view_state() == MONTH_VIEW, s);

  /* draw weekday */
  gc = gdk_gc_new(calendar->window);

  for(i = 0; i < 7; i++){
    gdk_gc_set_foreground (gc, &d_blue);
    gdk_draw_rectangle(pixmap, gc, TRUE, i * w + START_CALX, 16, w, 16);
    
    sprintf(s, "%s", _(weekday_name[i]));
    g = (w - gdk_string_width(font, s)) / 2 + START_CALX;

#if 0
    gdk_gc_set_foreground (gc, &d_glay);
    gdk_draw_string(pixmap, font, gc, i * w + g + 1, 16 + a + 1, s);
#endif

    gdk_draw_string(pixmap, font, calendar->style->white_gc, 
		    i * w + g, 16 + a, s);
  }

  wday = get_start_dayweek(year, month);
  mday = get_day_of_month(year, month);

  if(month >= 2)  pday = get_day_of_month(year, month - 1);
  else            pday = get_day_of_month(year - 1, 12);

  if(month <= 11) nday = get_day_of_month(year, month + 1);
  else            nday = get_day_of_month(year + 1, 1);

  /* draw prev month days */
  for(i = 0; i < wday; i++){
    draw_days(i, (pday - wday) + i + 1, &glay, 0, 0);
  }

  /* draw days */
  for(i = 0; i < mday; i++){
    am = c_state[i + 1] & 1;
    pm = (c_state[i + 1] >> 1) & 1;

    if(day == (i + 1) && (get_view_state() == DAY_VIEW)){
      gdk_gc_set_foreground (gc, &green);
      gdk_draw_rectangle(pixmap, gc, TRUE,
			 ((wday + i) % 7) * w + 1 + START_CALX,
			 ((wday + i) / 7) * h + 1 + START_CALY,
			 w - 2, h - 2);
      draw_days(wday + i, i + 1, &white, am, pm);
    }else if(is_holiday(year, month, i + 1) > 0)
      draw_days(wday + i, i + 1, &red, am, pm);
    else if((c_state[i + 1] >> 3) & 1)
      draw_days(wday + i, i + 1, &cyan, am, pm);
    else
      draw_days(wday + i, i + 1, &black, am, pm);
  }

  /* draw next month days */
  for(i = mday + wday; i < 42; i++){
    draw_days(i, i - (mday + wday) + 1, &glay, 0, 0);
  }

  i = 0, j = (get_view_state() == WEEK_VIEW);
  draw_week_button(0, i * h + START_CALY, START_CALX, h, 
	      j & (current_week == i ? 1 : 0), "1w");i++;
  draw_week_button(0, i * h + START_CALY, START_CALX, h, 
	      j & (current_week == i ? 1 : 0), "2w");i++;
  draw_week_button(0, i * h + START_CALY, START_CALX, h, 
	      j & (current_week == i ? 1 : 0), "3w");i++;
  draw_week_button(0, i * h + START_CALY, START_CALX, h, 
	      j & (current_week == i ? 1 : 0), "4w");i++;
  draw_week_button(0, i * h + START_CALY, START_CALX, h,
	      j & (current_week == i ? 1 : 0), "5w");i++;
  draw_week_button(0, i * h + START_CALY, START_CALX, h,
	      j & (current_week == i ? 1 : 0), "6w");i++;

  gdk_gc_destroy(gc);
  
  gtk_widget_draw(calendar, NULL);
}

int configure_event(GtkWidget *widget, GdkEventConfigure *event)
{
  if(pixmap){
    gdk_pixmap_unref(pixmap);
  }

  if(!left){
    left = gdk_pixmap_create_from_xpm_d(widget->window,
					&left_mask, NULL, leftarrow);
  }

  if(!right){
    right = gdk_pixmap_create_from_xpm_d(widget->window,
					 &right_mask, NULL, rightarrow);
  }

  if(!right_green){
    right_green = gdk_pixmap_create_from_xpm_d(widget->window,
					   &right_green_mask, NULL, 
					   arrow_gr_r);
  }

  if(!right_glay){
    right_glay = gdk_pixmap_create_from_xpm_d(widget->window,
					   &right_glay_mask, NULL, 
					   arrow_gl_r);
  }

  pixmap = gdk_pixmap_new(widget->window, 
			  event->width - ((event->width - START_CALX)% 7),
			  widget->allocation.height,
			  -1);

  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);
  draw_calendar(current_year, current_month, current_day);

  return TRUE;
}

int expose_event(GtkWidget *widget, GdkEventExpose *event)
{
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		  pixmap,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);

  return FALSE;
}

void set_date_by_pos(int x, int y)
{
  int day;

  day = x + y * 7 - get_start_dayweek(current_year, current_month) + 1;
  if(1 <= day && day <= get_day_of_month(current_year, current_month)){
    current_day = day;
  }else if(day < 1){
    current_month--;
    if(current_month < 1) current_month = 12, current_year--;
    current_day =  get_day_of_month(current_year, current_month) + day;

    current_week = (current_day + get_start_dayweek(current_year, current_month)) / 7;

    sched_destroy_list(sc_current);      
    sc_current = sc_previous;
    sc_previous = get_sched_previous_list(current_year, current_month);

  }else if(day > get_day_of_month(current_year, current_month)){
    current_day = day - get_day_of_month(current_year, current_month);
    current_month++;
    if(current_month > 12 && current_year < 2037)
      current_month = 1, current_year++;

    current_week = (current_day + get_start_dayweek(current_year, current_month)) / 7;

    sched_destroy_list(sc_previous);      
    sc_previous = sc_current;
    sc_current = get_sched_list(current_year, current_month);
  }
}

int year_next  = 0;
int year_prev  = 0;
int month_next = 0;
int month_prev = 0;

void next_press(int i)
{
  if(i){
    current_month++;
    if(current_month > 12){
      current_month = 1;
      current_year++;
    }
    
    if(current_year > 2037){
      current_year = 2037, current_month = 12;
    }else{
      sched_destroy_list(sc_previous);      
      sc_previous = sc_current;
      sc_current = get_sched_list(current_year, current_month);
      
      if(current_day > get_day_of_month(current_year, current_month)){
	current_day = get_day_of_month(current_year, current_month);
      }
    }
  }else{
    current_year++;
    if(current_year > 2037){
      current_year = 2037;
    }else{
      sched_destroy_list(sc_previous);      
      sc_previous = sc_current;
      sc_current = get_sched_list(current_year, current_month);
    
      if(current_day > get_day_of_month(current_year, current_month)){
	current_day = get_day_of_month(current_year, current_month);
      }
    }
  }

  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);
  draw_calendar(current_year, current_month, current_day);
}

void prev_press(int i)
{
  if((gint)i){
    current_month--;
    if(current_month < 1){
      current_month = 12;
      current_year--;
    }

    if(current_year < 2001){
      current_year = 2001, current_month = 1;
    }else{
      sched_destroy_list(sc_current);      
      sc_current = sc_previous;
      sc_previous = get_sched_previous_list(current_year, current_month);
      
      if(current_day > get_day_of_month(current_year, current_month)){
	current_day = get_day_of_month(current_year, current_month);
      }
    }

  }else{
    current_year--;
    if(current_year < 2001){
      current_year = 2001;
    }else{
      sched_destroy_list(sc_current);      
      sc_current = sc_previous;
      sc_previous = get_sched_previous_list(current_year, current_month);

      if(current_day > get_day_of_month(current_year, current_month)){
	current_day = get_day_of_month(current_year, current_month);
      }
    }
  }

  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);
  draw_calendar(current_year, current_month, current_day);
}

void button_press(GtkWidget *widget, GdkEventButton *event)
{
  int  x, y;
  int  h, w;
  int  m;
  GdkGC *gc;
  GdkRectangle rect;

  static int old = 0;

  w = (calendar->allocation.width -  START_CALX) / 7;
  h = (calendar->allocation.height - START_CALY) / 6;

  x = (int) event->x;
  y = (int) event->y;

  if((y > START_CALY)&&(event->time - old < 200)){
    old = event->time;
    
    set_sched_data_now();
    set_ok_btn_state(0);
    set_page(1);
    return;
  }

  old = event->time;

  m = calendar->allocation.width - ((calendar->allocation.width - START_CALX) % 7);

  if((y > START_CALY)){
    if(x > START_CALX){
      current_week = (y - START_CALY)/ h;
      x = (x - START_CALX)/ w;
      y = (y - START_CALY) / h;
      set_date_by_pos(x, y);
      view_press(NULL, (gpointer) DAY_VIEW);
    }else{
      current_week = (y - START_CALY) / h;
      if(current_week != 
	 (current_day + get_start_dayweek(current_year, current_month))/7){
	current_day = current_week * 7 - get_start_dayweek(current_year, current_month) + 1;

	if(current_day > get_day_of_month(current_year, current_month)){
	  current_day -= get_day_of_month(current_year, current_month);
	  current_month++;
	  if(current_month > 12)
	    current_year++, current_month = 1;

	  if(current_year > 2037)
	    current_year = 2037, current_month = 12, current_day = 31;

	  current_week = (current_day + get_start_dayweek(current_year, current_month)) / 7;

	  sched_destroy_list(sc_previous);      
	  sc_previous = sc_current;
	  sc_current = get_sched_list(current_year, current_month);
	}else if(current_day < 1){
	  current_day = 1;
	  current_week = 0;
	}
      }

      view_press(NULL, (gpointer) WEEK_VIEW);
    }
  }else if(y < h){
    if(x < START_CALX){
      year_prev = 1;

      gc = gdk_gc_new(calendar->window);
      gdk_gc_set_foreground (gc, &glay);
      gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, ARROW_WIDTH, 14);
      gdk_gc_destroy(gc);
      draw_left_arrow(1, 1);

      rect.x = 0;
      rect.y = 0;
      rect.width = ARROW_WIDTH;
      rect.height = 14;
      gtk_widget_draw(calendar, &rect);
    }else if(x >= (ARROW_WIDTH+48) && x < (ARROW_WIDTH*2+48)){
      year_next = 1;

      gc = gdk_gc_new(calendar->window);
      gdk_gc_set_foreground (gc, &glay);
      gdk_draw_rectangle(pixmap, gc, TRUE, 
			 (ARROW_WIDTH + 48), 0, ARROW_WIDTH, 14);
      gdk_gc_destroy(gc);
      
      draw_right_arrow(ARROW_WIDTH + 49, 1);

      rect.x = ARROW_WIDTH + 48;
      rect.y = 0;
      rect.width = ARROW_WIDTH;
      rect.height = 14;
      gtk_widget_draw(calendar, &rect);
    }else if(x >= m - (ARROW_WIDTH*2 + 48)&& x < m - (ARROW_WIDTH + 48)){          
      month_prev = 1;

      gc = gdk_gc_new(calendar->window);
      gdk_gc_set_foreground (gc, &glay);
      gdk_draw_rectangle(pixmap, gc, TRUE, 
			 m - (ARROW_WIDTH*2+48), 0, ARROW_WIDTH, 14);
      gdk_gc_destroy(gc);
      draw_left_arrow(m - (ARROW_WIDTH*2 + 47), 1);

      rect.x = m - (ARROW_WIDTH*2 + 48);
      rect.y = 0;
      rect.width = ARROW_WIDTH;
      rect.height = 14;
      gtk_widget_draw(calendar, &rect);

    }else if(x >= m - ARROW_WIDTH && x < m){
      month_next = 1;

      gc = gdk_gc_new(calendar->window);
      gdk_gc_set_foreground (gc, &glay);
      gdk_draw_rectangle(pixmap, gc, TRUE, 
			 m - ARROW_WIDTH, 0, ARROW_WIDTH, 14);
      gdk_gc_destroy(gc);
      draw_right_arrow(m - (ARROW_WIDTH - 1), 1);

      rect.x = m - ARROW_WIDTH;
      rect.y = 0;
      rect.width = ARROW_WIDTH;
      rect.height = 14;
      gtk_widget_draw(calendar, &rect);
    }else if(m - (ARROW_WIDTH*2 + 48)< x && x < m - ARROW_WIDTH){
      view_press(NULL, (gpointer) MONTH_VIEW);
    }
  }
}

void button_release(GtkWidget *widget, GdkEventButton *event)
{
  int  x, y;
  int  h, w, g, a;
  int  m;

  char s[256];
  GdkGC *gc;
  GdkRectangle rect;

  w = (calendar->allocation.width -  START_CALX)/ 7;

  a = font->ascent;
  h = (calendar->allocation.height - START_CALY) / 6;

  x = (int) event->x;
  y = (int) event->y;

  m = calendar->allocation.width -((calendar->allocation.width - START_CALX) % 7);

  if(y < h && !(x > (ARROW_WIDTH*2 + 48) && x < m - (ARROW_WIDTH*2 + 48))){
    if(x < START_CALX && year_prev){
      year_prev = 0;
      prev_press(0);
    }else if(x >= (ARROW_WIDTH + 48) && x < (ARROW_WIDTH*2 + 48) && year_next){
      year_next = 0;
      next_press(0);
    }else if((x >= (m - (ARROW_WIDTH*2 + 48))) && (x < (m - (ARROW_WIDTH + 48))) && month_prev){
      month_prev = 0;
      prev_press(1);
    }else if((x >= (m - ARROW_WIDTH)) && (x < m) && month_next){
      month_next = 0;
      next_press(1);
    }
  }else{
    gc = calendar->style->white_gc;
    gdk_draw_rectangle(pixmap, gc, TRUE,
		       0, 0, m, 14);
    
    gc = gdk_gc_new(calendar->window);
    gdk_gc_set_foreground (gc, &glay);
    gdk_draw_rectangle(pixmap, gc, TRUE,
		       0, 0, m, h);
    gdk_gc_destroy(gc);
    
    draw_left_arrow(0, 0);
    draw_right_arrow(ARROW_WIDTH + 48, 0);
    
    sprintf(s, "%04d", current_year);
    g = (48 - gdk_string_width(calendar->style->font, s))/2 + START_CALX;
    gc = calendar->style->black_gc;
    gdk_draw_string(pixmap, font, gc, g, a, s);
    
    draw_left_arrow(m - (ARROW_WIDTH*2 + 48), 0);
    draw_right_arrow(m - ARROW_WIDTH, 0);

    sprintf(s, "%s", _(month_name[(current_month - 1)]));
    g = (48 - gdk_string_width(calendar->style->font, s))/2;
    draw_month_button(m - (ARROW_WIDTH + 48), 0, 46, 14,
		      get_view_state() == MONTH_VIEW, s);

    rect.x = 0;
    rect.y = 0;
    rect.width = m;
    rect.height = 14;

    draw_calendar(current_year, current_month, current_day);
    gtk_widget_draw(calendar, &rect);
  }
}

GtkWidget *
init_calendar()
{
  int h, w;
  GdkColormap *colormap;
  
  calendar = gtk_drawing_area_new();
  gtk_widget_set_events(calendar, 
			gtk_widget_get_events(calendar) 
			| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
			);

  font = gdk_fontset_load("-misc-fixed-medium-r-normal--14-*-*-*-*-*");

  w = (calendar->allocation.width - START_CALX)/ 7;
  h = font->ascent + font->descent;

  gtk_signal_connect (GTK_OBJECT (calendar), "configure_event",
		      GTK_SIGNAL_FUNC (configure_event), NULL);
  gtk_signal_connect (GTK_OBJECT (calendar), "expose_event",
		      GTK_SIGNAL_FUNC (expose_event), NULL);
  gtk_signal_connect (GTK_OBJECT (calendar), "button_press_event",
		      GTK_SIGNAL_FUNC (button_press), NULL);
  gtk_signal_connect (GTK_OBJECT (calendar), "button_release_event",
		      GTK_SIGNAL_FUNC (button_release), NULL);

  gtk_drawing_area_size(GTK_DRAWING_AREA(calendar),
			-1, h * 6 + START_CALY);

  colormap = gtk_widget_get_colormap(calendar);

  gdk_color_alloc (colormap, &glay);
  gdk_color_alloc (colormap, &l_glay);
  gdk_color_alloc (colormap, &d_glay);
  gdk_color_alloc (colormap, &black);
  gdk_color_alloc (colormap, &white);
  gdk_color_alloc (colormap, &red);
  gdk_color_alloc (colormap, &blue);
  gdk_color_alloc (colormap, &green);
  gdk_color_alloc (colormap, &d_red);
  gdk_color_alloc (colormap, &d_blue);
  gdk_color_alloc (colormap, &d_green);
  gdk_color_alloc (colormap, &cyan);
      
  return calendar;
}
