/* 
   $Id: sgsched_edit.c,v 1.7 2001/12/27 08:21:16 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgsched.

   sgsched is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgsched is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgsched; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sgsched.h"

GtkWidget *start_year;
GtkWidget *start_mon;
GtkWidget *start_day;
GtkWidget *start_hour;
GtkWidget *start_min;

GtkWidget *end_year;
GtkWidget *end_mon;
GtkWidget *end_day;
GtkWidget *end_hour;
GtkWidget *end_min;

#ifdef STYLE_COMBO
GList     *years, *months, *days, *hours, *mins;
#else
GList     *locations, *persons;
#endif

GtkWidget *detail;
GtkWidget *location;
GtkWidget *person;

GtkWidget *normal, *weekly, *monthly, *holiday;
GSList    *group = NULL;

GtkWidget  *ok_btn;
static int ok_btn_state = 0;

GtkWidget *spec_table;
GtkWidget *spec_year, *spec_mon, *spec_day;

extern int    current_year;
extern int    current_month;
extern int    current_day;
extern struct sched_data *sc_all;
extern struct sched_data *sc_current;
extern struct sched_data *sc_previous;

int get_ok_btn_state()
{
  return ok_btn_state;
}

void set_ok_btn_state(int i)
{
  switch(i){
  case 0:
    ok_btn_state = 0;
    gtk_label_set_text(GTK_LABEL(GTK_BUTTON(ok_btn)->child), _("Add"));
    break;

  case 1:
    ok_btn_state = 1;
    gtk_label_set_text(GTK_LABEL(GTK_BUTTON(ok_btn)->child), _("Update"));
    break;

  case 2:
    ok_btn_state = 2;
    gtk_label_set_text(GTK_LABEL(GTK_BUTTON(ok_btn)->child), _("Delete"));
    break;
  default:break;
  }

}

int get_entry_num(GtkWidget *widget)
{
  char *s;

  s = gtk_entry_get_text(GTK_ENTRY(widget));
  if(s) return atoi(s);

  return -1;
}

void set_entry_num(GtkWidget *widget, int num)
{
  char s[256];
  sprintf(s, "%d", num);
  gtk_entry_set_text(GTK_ENTRY(widget), s);
}

void set_sched_data(struct sched_data *data)
{
  struct tm *tm;
  
  tm = localtime(&data->start);
  if(tm){
#ifdef STYLE_COMBO
    set_entry_num(GTK_COMBO(start_year)->entry, tm->tm_year + 1900);
    set_entry_num(GTK_COMBO(start_mon)->entry,  tm->tm_mon + 1);
    set_entry_num(GTK_COMBO(start_day)->entry,  tm->tm_mday);
    set_entry_num(GTK_COMBO(start_hour)->entry, tm->tm_hour);
    set_entry_num(GTK_COMBO(start_min)->entry,  tm->tm_min);
#else
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(start_year), (gfloat) tm->tm_year + 1900);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(start_mon),  (gfloat) tm->tm_mon + 1);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(start_day),  (gfloat) tm->tm_mday);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(start_hour), (gfloat) tm->tm_hour);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(start_min),  (gfloat) tm->tm_min);
#endif
  }

  tm = localtime(&data->end);
  if(tm){
#ifdef STYLE_COMBO
    set_entry_num(GTK_COMBO(end_year)->entry, tm->tm_year + 1900);
    set_entry_num(GTK_COMBO(end_mon)->entry,  tm->tm_mon + 1);
    set_entry_num(GTK_COMBO(end_day)->entry,  tm->tm_mday);
    set_entry_num(GTK_COMBO(end_hour)->entry, tm->tm_hour);
    set_entry_num(GTK_COMBO(end_min)->entry,  tm->tm_min);
#else
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(end_year), (gfloat) tm->tm_year + 1900);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(end_mon),  (gfloat) tm->tm_mon + 1);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(end_day),  (gfloat) tm->tm_mday);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(end_hour), (gfloat) tm->tm_hour);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(end_min),  (gfloat) tm->tm_min);
#endif
  }

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(normal), TRUE);

  if(data->flag & WEEKLY_SCHEDULE)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(weekly), TRUE);
  if(data->flag & MONTHLY_SCHEDULE)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(monthly), TRUE);

  if(data->flag & DAYOFF_SCHEDULE)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(holiday), TRUE);
  else
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(holiday), FALSE);

  gtk_editable_delete_text(GTK_EDITABLE(detail), 0, -1);

  if(data->location)
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(location)->entry), data->location);
  else
    gtk_editable_delete_text(GTK_EDITABLE(GTK_COMBO(location)->entry), 0, -1);

  if(data->person)
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(person)->entry), data->person);
  else
    gtk_editable_delete_text(GTK_EDITABLE(GTK_COMBO(person)->entry),
			     0, -1);

  if(data->data){
    gtk_text_insert(GTK_TEXT(detail), NULL, NULL, NULL,
		    data->data, strlen(data->data));
  }

}

void ok_press(GtkWidget *widget, gpointer data)
{
  struct tm  tm;
  char       *p = NULL;
  char       *home;
  char       dname[BUFSIZ];
  char       fn[BUFSIZ];
  int        i, j;
  DIR        *dirp;
  struct sched_data *sc_data;

  switch(ok_btn_state){
  case 0:
    sc_data = (struct sched_data *) calloc(1, sizeof(struct sched_data));
    
    sc_data->time = time(NULL);

#ifdef STYLE_COMBO
    tm.tm_year     = get_entry_num(GTK_COMBO(start_year)->entry) - 1900;
    tm.tm_mon      = get_entry_num(GTK_COMBO(start_mon)->entry)  - 1;
    tm.tm_mday     = get_entry_num(GTK_COMBO(start_day)->entry);
    tm.tm_hour     = get_entry_num(GTK_COMBO(start_hour)->entry);
    tm.tm_min      = get_entry_num(GTK_COMBO(start_min)->entry);
#else
    tm.tm_year     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_year)) - 1900;
    tm.tm_mon      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_mon))  - 1;
    tm.tm_mday     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_day));
    tm.tm_hour     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_hour));
    tm.tm_min      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_min));
#endif
    tm.tm_sec      = 0;
    tm.tm_isdst    = -1;
    sc_data->start = mktime(&tm);
#ifdef STYLE_COMBO
    tm.tm_year   = get_entry_num(GTK_COMBO(end_year)->entry) - 1900;
    tm.tm_mon    = get_entry_num(GTK_COMBO(end_mon)->entry)  - 1;
    tm.tm_mday   = get_entry_num(GTK_COMBO(end_day)->entry);
    tm.tm_hour   = get_entry_num(GTK_COMBO(end_hour)->entry);
    tm.tm_min    = get_entry_num(GTK_COMBO(end_min)->entry);
#else    
    tm.tm_year   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_year)) - 1900;
    tm.tm_mon    = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_mon))  - 1;
    tm.tm_mday   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_day));
    tm.tm_hour   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_hour));
    tm.tm_min    = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_min));
#endif
    tm.tm_sec    = 0;
    tm.tm_isdst  = -1;
    sc_data->end = mktime(&tm);

    if(sc_data->start > sc_data->end)
      sc_data->end = sc_data->start;

    sc_data->flag = 0;

    i = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(normal));
    if(i) sc_data->flag = NORMAL_SCHEDULE;
    i = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(weekly));
    if(i) sc_data->flag = WEEKLY_SCHEDULE;
    i = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(monthly));
    if(i) sc_data->flag = MONTHLY_SCHEDULE;

    j = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(holiday));
    if(j)
      sc_data->flag |= DAYOFF_SCHEDULE;
    else
      sc_data->flag &= ~DAYOFF_SCHEDULE;

    p = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(location)->entry));
    if(p && strlen(p) > 0){
      sc_data->location = strdup(p);
      if(g_list_find_custom(locations, p, (GCompareFunc)strcmp) == NULL){
	locations = g_list_append(locations, (gpointer)strdup(p));
	gtk_combo_set_popdown_strings(GTK_COMBO(location), locations);
      }
    }

    p = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(person)->entry));
    if(p && strlen(p) > 0){
      sc_data->person = strdup(p);
      if(g_list_find_custom(persons, p, (GCompareFunc)strcmp) == NULL){
	persons = g_list_append(persons, (gpointer)strdup(p));
	gtk_combo_set_popdown_strings(GTK_COMBO(person), persons);
      }
    }

    p = gtk_editable_get_chars(GTK_EDITABLE(detail), 0, 
				 gtk_text_get_length(GTK_TEXT(detail)));
    
    if(p && strlen(p) > 0)  sc_data->data = strdup(p);

    sched_add_data(sc_data);
    break;

  case 1:
    sc_data = get_selected_row_data();

    sc_data->time = time(NULL);
#ifdef STYLE_COMBO    
    tm.tm_year     = get_entry_num(GTK_COMBO(start_year)->entry) - 1900;
    tm.tm_mon      = get_entry_num(GTK_COMBO(start_mon)->entry)  - 1;
    tm.tm_mday     = get_entry_num(GTK_COMBO(start_day)->entry);
    tm.tm_hour     = get_entry_num(GTK_COMBO(start_hour)->entry);
    tm.tm_min      = get_entry_num(GTK_COMBO(start_min)->entry);
#else
    tm.tm_year     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_year)) - 1900;
    tm.tm_mon      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_mon))  - 1;
    tm.tm_mday     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_day));
    tm.tm_hour     = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_hour));
    tm.tm_min      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_min));
#endif
    tm.tm_sec      = 0;
    tm.tm_isdst    = -1;
    sc_data->start = mktime(&tm);
#ifdef STYLE_COMBO
    tm.tm_year   = get_entry_num(GTK_COMBO(end_year)->entry) - 1900;
    tm.tm_mon    = get_entry_num(GTK_COMBO(end_mon)->entry)  - 1;
    tm.tm_mday   = get_entry_num(GTK_COMBO(end_day)->entry);
    tm.tm_hour   = get_entry_num(GTK_COMBO(end_hour)->entry);
    tm.tm_min    = get_entry_num(GTK_COMBO(end_min)->entry);
#else    
    tm.tm_year   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_year)) - 1900;
    tm.tm_mon    = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_mon))  - 1;
    tm.tm_mday   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_day));
    tm.tm_hour   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_hour));
    tm.tm_min    = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_min));
#endif
    tm.tm_sec    = 0;
    tm.tm_isdst  = -1;
    sc_data->end = mktime(&tm);

    if(sc_data->start > sc_data->end)
      sc_data->end = sc_data->start;

    sc_data->flag = 0;
        
    i = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(normal));
    if(i) sc_data->flag = NORMAL_SCHEDULE;
    i = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(weekly));
    if(i) sc_data->flag = WEEKLY_SCHEDULE;
    i = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(monthly));
    if(i) sc_data->flag = MONTHLY_SCHEDULE;

    j = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(holiday));
    if(j)
      sc_data->flag |= DAYOFF_SCHEDULE;
    else
      sc_data->flag &= ~DAYOFF_SCHEDULE;

    p = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(location)->entry));
    if(p && strlen(p) > 0){
      sc_data->location = strdup(p);
      if(g_list_find_custom(locations, p, (GCompareFunc)strcmp) == NULL){
	locations = g_list_append(locations, (gpointer)strdup(p));
	gtk_combo_set_popdown_strings(GTK_COMBO(location), locations);
      }
    }

    p = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(person)->entry));
    if(p && strlen(p) > 0){
      sc_data->person = strdup(p);
      if(g_list_find_custom(persons, p, (GCompareFunc)strcmp) == NULL){
	persons = g_list_append(persons, (gpointer)strdup(p));
	gtk_combo_set_popdown_strings(GTK_COMBO(person), persons);
      }
    }

    p = gtk_editable_get_chars(GTK_EDITABLE(detail), 0, 
			       gtk_text_get_length(GTK_TEXT(detail)));
    
    if(p && strlen(p) > 0)  sc_data->data = strdup(p);

    set_ok_btn_state(0);
    break;

  case 2:
    sc_data = get_selected_row_data();
    sched_del_data(sc_data);
    set_ok_btn_state(0);
    break;

  default:break;
  }
  
  if ((home = getenv("HOME")) != NULL) {
    sprintf(dname, "%s/sgsched",home);
  } else {
    sprintf(dname, "sgsched");
  }
  
  dirp = opendir(dname);

  if(!dirp) {
    mkdir(dname,0700);
  } else      closedir(dirp);

  sprintf(fn,"%s/schedule", dname);
  sched_save_list(fn);
  
  if(sc_current) sched_destroy_list(sc_current);      
  sc_current = get_sched_list(current_year, current_month);
  if(sc_previous) sched_destroy_list(sc_previous);      
  sc_previous = get_sched_previous_list(current_year, current_month);

  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);
  draw_calendar(current_year, current_month, current_day);

  set_page(0);
}

void cancel_press(GtkWidget *widget, gpointer data)
{
  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);
  draw_calendar(current_year, current_month, current_day);

  set_page(0);
}

void check_selected_day(GtkWidget *widget, gpointer data)
{
  int year = 0, month = 0, day = 0;

  if((int)data){ 
#ifdef STYLE_COMBO
    year  = get_entry_num(GTK_COMBO(start_year)->entry);
    month = get_entry_num(GTK_COMBO(start_mon)->entry);
    day   = get_entry_num(GTK_COMBO(start_day)->entry);
#else
    year  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_year));
    month = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_mon));
    day   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(start_day));
#endif
    if(day > get_day_of_month(year, month)){
#ifdef STYLE_COMBO
      set_entry_num(GTK_COMBO(start_day)->entry,
		    get_day_of_month(year,month));
#else
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(start_day), (gfloat)get_day_of_month(year,month));
#endif
    }
  }else{
#ifdef STYLE_COMBO
    year  = get_entry_num(GTK_COMBO(end_year)->entry);
    month = get_entry_num(GTK_COMBO(end_mon)->entry);
    day   = get_entry_num(GTK_COMBO(end_day)->entry);
#else
    year  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_year));
    month = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_mon));
    day   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(end_day));
#endif
    if(day > get_day_of_month(year, month)){
#ifdef STYLE_COMBO
      set_entry_num(GTK_COMBO(end_day)->entry,
		    get_day_of_month(year,month));
#else
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(end_day), (gfloat)get_day_of_month(year,month));
#endif
    }
  }
}

void load_location_data()
{
  struct sched_data *tmp;

  if(sc_all == NULL) return;

  tmp = sc_all->next;
  while(tmp){
    if(tmp->location != NULL){
      if(g_list_find_custom(locations, tmp->location, (GCompareFunc)strcmp) == NULL)
	locations = g_list_append(locations, (gpointer)strdup(tmp->location));
    }
    tmp = tmp->next;
  }
}

void load_person_data()
{
  FILE               *fp = NULL;
  struct sched_data  *tmp;
  char               str[4096];
  char               *home;
  char               fn[4096];

  char *s;
  int  i,c;

  if(sc_all == NULL) return;

  tmp = sc_all->next;
  while(tmp){
    if(tmp->person != NULL)
      if(g_list_find_custom(persons, tmp->person, (GCompareFunc)strcmp) == NULL)
	persons = g_list_append(persons, (gpointer)strdup(tmp->person));
    tmp = tmp->next;
  }

  if ((home = getenv("HOME")) != NULL) {
    sprintf(fn,"%s/sgaddrbook",home);
  } else {
    sprintf(fn,"sgaddrbook");
  }
  fp = fopen(fn, "r");
  
  if(fp == NULL){
    fprintf(stderr, "I can't open address book file:%s.\n"
      "I will try to create one later.\n",fn);
    return;
  }

  s = str;
  i = 0;
  while((c = fgetc(fp)) != EOF){
    switch(c){
    case '(': 
      i = 0;
      s = str;
      break;

    case ')':
      *s = '\0';
      i = 0;
      break;

    case '\n':
      break;

    case '\"':
      while( (c = fgetc(fp)) != EOF){
	if(c == '\"') break;
	*s++ = c;
      }
      break;

    case ' ':
      *s = '\0';
      if(i == 0 && strlen(str) > 0){
	if(g_list_find_custom(persons, str, (GCompareFunc)strcmp) == NULL)
	  persons = g_list_append(persons, (gpointer)strdup(str));
      }

      i++;
      s = str;
      break;
      
    default:
      *s++ = c;
      break;
    }
  }

  fclose(fp);
}

void create_combo(GtkWidget *edit_table)
{
  GtkWidget *label;
  GtkAdjustment *adjustment;
#ifdef STYLE_COMBO
  char s[256];
  
  years = NULL;
  for(i = 1; i <= 37; i++){
    sprintf(s, "%04d", i + 2000);
    years = g_list_append(years, strdup(s));
  }

  months = NULL;
  for(i = 1; i <= 12; i++){
    sprintf(s, "%2d", i);
    months = g_list_append(months, strdup(s));
  }

  days = NULL;
  for(i = 1; i <= 31; i++){
    sprintf(s, "%2d", i);
    days = g_list_append(days, strdup(s));
  }

  hours = NULL;
  for(i = 0; i < 24; i++){
    sprintf(s, "%2d", i);
    hours = g_list_append(hours, strdup(s));
  }

  mins = NULL;
  for(i = 0; i < 60; i++){
    sprintf(s, "%2d", i);
    mins = g_list_append(mins, strdup(s));
  }
#endif

  label = gtk_label_new(_("Start of Schedule"));
  gtk_table_attach(GTK_TABLE(edit_table), label, 0, 10, 0, 1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);
#ifdef STYLE_COMBO
  start_year = gtk_combo_new();
  start_mon  = gtk_combo_new();
  start_day  = gtk_combo_new();
  start_hour = gtk_combo_new();
  start_min  = gtk_combo_new();
#else
  adjustment = (GtkAdjustment *) gtk_adjustment_new (2001.0 , 2001.0 , 2037.0, 1.0, 25.0, 25.0);
  start_year = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (1.0, 1.0 , 12.0, 1.0, 25.0, 25.0);
  start_mon  = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (1.0, 1.0 , 31.0, 1.0, 25.0, 25.0);
  start_day  = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0 , 23.0, 1.0, 25.0, 25.0);
  start_hour = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0 , 59.0, 1.0, 25.0, 25.0);
  start_min  = gtk_spin_button_new(adjustment, 1.0, 0);
#endif

  gtk_table_attach(GTK_TABLE(edit_table), start_year, 0, 2, 2, 3,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), start_mon,  2, 4, 2, 3,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND, 
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), start_day,  4, 6, 2, 3,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), start_hour, 6, 8, 2, 3,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), start_min,  8, 10, 2, 3,
		   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);

#ifdef STYLE_COMBO
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(start_year)->entry), "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)1);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(start_mon)->entry),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)1);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(start_day)->entry),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)1);

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(start_year)->entry), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(start_mon)->entry),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(start_day)->entry),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(start_hour)->entry), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(start_min)->entry),  FALSE);

  gtk_combo_set_popdown_strings(GTK_COMBO(start_year), years);
  gtk_combo_set_popdown_strings(GTK_COMBO(start_mon),  months);
  gtk_combo_set_popdown_strings(GTK_COMBO(start_day),  days);
  gtk_combo_set_popdown_strings(GTK_COMBO(start_hour), hours);
  gtk_combo_set_popdown_strings(GTK_COMBO(start_min),  mins);

  gtk_widget_set_usize(start_year, 50, -1);
  gtk_widget_set_usize(start_mon,  36, -1);
  gtk_widget_set_usize(start_day,  36, -1);
  gtk_widget_set_usize(start_hour, 36, -1);
  gtk_widget_set_usize(start_min,  36, -1);
#else
  gtk_signal_connect(GTK_OBJECT(start_year), "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)1);
  gtk_signal_connect(GTK_OBJECT(start_mon),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)1);
  gtk_signal_connect(GTK_OBJECT(start_day),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)1);
  gtk_entry_set_editable(GTK_ENTRY(start_year), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(start_mon),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(start_day),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(start_hour), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(start_min),  FALSE);

  gtk_widget_set_usize(start_year, 50, -1);
  gtk_widget_set_usize(start_mon,  36, -1);
  gtk_widget_set_usize(start_day,  36, -1);
  gtk_widget_set_usize(start_hour, 36, -1);
  gtk_widget_set_usize(start_min,  36, -1);

#endif

  label = gtk_label_new(_("End of Schedule"));
  gtk_table_attach(GTK_TABLE(edit_table), label, 0, 10, 3, 4,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);
#ifdef STYLE_COMBO
  end_year = gtk_combo_new();
  end_mon  = gtk_combo_new();
  end_day  = gtk_combo_new();
  end_hour = gtk_combo_new();
  end_min  = gtk_combo_new();
#else
  adjustment = (GtkAdjustment *) gtk_adjustment_new (2001.0 , 2001.0 , 2037.0, 1.0, 25.0, 25.0);
  end_year = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (1.0, 1.0 , 12.0, 1.0, 25.0, 25.0);
  end_mon  = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (1.0, 1.0 , 31.0, 1.0, 25.0, 25.0);
  end_day  = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0 , 23.0, 1.0, 25.0, 25.0);
  end_hour = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0 , 59.0, 1.0, 25.0, 25.0);
  end_min  = gtk_spin_button_new(adjustment, 1.0, 0);
#endif
  gtk_table_attach(GTK_TABLE(edit_table), end_year, 0, 2, 4, 5,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), end_mon,  2, 4, 4, 5,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), end_day,  4, 6, 4, 5,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), end_hour, 6, 8, 4, 5,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
  gtk_table_attach(GTK_TABLE(edit_table), end_min,  8, 10, 4, 5,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK, 0, 0);
#ifdef STYLE_COMBO
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(end_year)->entry), "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)0);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(end_mon)->entry),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)0);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(end_day)->entry),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)0);

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(end_year)->entry), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(end_mon)->entry),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(end_day)->entry),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(end_hour)->entry), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(end_min)->entry),  FALSE);
  
  gtk_widget_set_usize(end_year, 50, -1);
  gtk_widget_set_usize(end_mon,  36, -1);
  gtk_widget_set_usize(end_day,  36, -1);
  gtk_widget_set_usize(end_hour, 36, -1);
  gtk_widget_set_usize(end_min,  36, -1);
  
  gtk_combo_set_popdown_strings(GTK_COMBO(end_year), years);
  gtk_combo_set_popdown_strings(GTK_COMBO(end_mon),  months);
  gtk_combo_set_popdown_strings(GTK_COMBO(end_day),  days);
  gtk_combo_set_popdown_strings(GTK_COMBO(end_hour), hours);
  gtk_combo_set_popdown_strings(GTK_COMBO(end_min),  mins);
#else
  gtk_signal_connect(GTK_OBJECT(end_year), "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)0);
  gtk_signal_connect(GTK_OBJECT(end_mon),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)0);
  gtk_signal_connect(GTK_OBJECT(end_day),  "changed",
		     GTK_SIGNAL_FUNC(check_selected_day), (gpointer)0);

  gtk_entry_set_editable(GTK_ENTRY(end_year), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(end_mon),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(end_day),  FALSE);
  gtk_entry_set_editable(GTK_ENTRY(end_hour), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(end_min),  FALSE);

  gtk_widget_set_usize(end_year, 50, -1);
  gtk_widget_set_usize(end_mon,  36, -1);
  gtk_widget_set_usize(end_day,  36, -1);
  gtk_widget_set_usize(end_hour, 36, -1);
  gtk_widget_set_usize(end_min,  36, -1);
#endif

}

void init_edit(GtkWidget *notebook)
{
  GtkWidget *edit_table;
  GtkWidget *toggle_table;

  GtkWidget *label;
  GtkWidget *cancel_btn;

  GtkWidget *text_table;
  GtkWidget *vscroll;

#ifdef __SIKIGAMI_PDA__
#else
  GtkWidget *edit_table2, *edit_table3;
#endif

  edit_table = gtk_table_new(15, 10,  FALSE);
#ifdef __SIKIGAMI_PDA__
#else
  edit_table2 = gtk_table_new(15, 10,  FALSE);
  edit_table3 = gtk_table_new(15, 10,  FALSE);

  gtk_table_attach(GTK_TABLE(edit_table), edit_table2, 0, 5, 0, 15,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);
  
  gtk_table_attach(GTK_TABLE(edit_table), edit_table3, 5, 10, 0, 15,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);
#endif
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), edit_table, NULL);

#ifdef __SIKIGAMI_PDA__
#else
  edit_table = edit_table3;
#endif
  ok_btn = gtk_button_new_with_label(_("Add"));
  gtk_table_attach(GTK_TABLE(edit_table), ok_btn, 0, 5, 14, 15,
                   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
  gtk_signal_connect(GTK_OBJECT(ok_btn),  "clicked",
		     GTK_SIGNAL_FUNC(ok_press), NULL);

  cancel_btn = gtk_button_new_with_label(_("Cancel"));
  gtk_table_attach(GTK_TABLE(edit_table), cancel_btn, 5, 10, 14, 15,
                   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
  gtk_signal_connect(GTK_OBJECT(cancel_btn),  "clicked",
		     GTK_SIGNAL_FUNC(cancel_press), NULL);

#ifdef __SIKIGAMI_PDA__
  create_combo(edit_table);
#else
  create_combo(edit_table2);
#endif

#ifndef __SIKIGAMI_PDA__
  edit_table = edit_table2;
#endif

  label = gtk_label_new(_("Type of Schedule"));
  gtk_table_attach(GTK_TABLE(edit_table), label, 0, 10, 5, 6,
                   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);

  toggle_table = gtk_table_new(2, 3,  FALSE);
  gtk_table_attach(GTK_TABLE(edit_table), toggle_table, 0, 10, 6, 7,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);

  group = NULL;
  normal = gtk_radio_button_new_with_label(group, _("Normal"));
  gtk_table_attach(GTK_TABLE(toggle_table), normal, 0, 1, 0, 1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);

  group = gtk_radio_button_group(GTK_RADIO_BUTTON(normal));

  /*  weekly = gtk_radio_button_new_with_label(group, _("Weekly"));*/
  weekly = gtk_radio_button_new_with_label(group, _("Week"));
  gtk_table_attach(GTK_TABLE(toggle_table), weekly, 1, 2, 0, 1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);

  group = gtk_radio_button_group(GTK_RADIO_BUTTON(weekly));

  /*  monthly = gtk_radio_button_new_with_label(group, _("Monthly"));*/
  monthly = gtk_radio_button_new_with_label(group, _("Month"));
  gtk_table_attach(GTK_TABLE(toggle_table), monthly, 2, 3, 0, 1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);

  /*  holiday = gtk_check_button_new_with_label(_("Holiday"));*/
  holiday = gtk_check_button_new_with_label(_("Off"));
  gtk_table_attach(GTK_TABLE(toggle_table), holiday, 3, 4, 0, 1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);

  label = gtk_label_new(_("Location"));
  gtk_table_attach(GTK_TABLE(edit_table), label, 0, 10, 7, 8,
                   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
  location = gtk_combo_new();
  gtk_table_attach(GTK_TABLE(edit_table), location, 0, 10, 8, 9,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);

  label = gtk_label_new(_("Person"));
  gtk_table_attach(GTK_TABLE(edit_table), label, 0, 10, 9, 10,
                   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
  load_location_data();
  if(locations != NULL)
    gtk_combo_set_popdown_strings(GTK_COMBO(location), locations);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(location)->entry),
		     "focus_in_event",
		     GTK_SIGNAL_FUNC(set_selection_src_widget), 
		     0);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(location)->entry),
		     "focus_out_event",
		     GTK_SIGNAL_FUNC(unset_selection_src_widget), 
		     0);


  person = gtk_combo_new();
  gtk_table_attach(GTK_TABLE(edit_table), person, 0, 10, 10, 11,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_SHRINK, 0, 0);
  load_person_data();
  if(persons != NULL)
    gtk_combo_set_popdown_strings(GTK_COMBO(person), persons);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(person)->entry), 
		     "focus_in_event",
		     GTK_SIGNAL_FUNC(set_selection_src_widget), 
		     0);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(person)->entry),
		     "focus_out_event",
		     GTK_SIGNAL_FUNC(unset_selection_src_widget), 
		     0);

  gtk_widget_set_usize(location, 180, -1);
  gtk_widget_set_usize(person,   180, -1);

#ifndef __SIKIGAMI_PDA__
  edit_table = edit_table3;
#endif
  label = gtk_label_new(_("Detail of Schedule"));
  gtk_table_attach(GTK_TABLE(edit_table), label, 0, 10, 11, 12,
                   GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);

  text_table = gtk_table_new(1, 2, FALSE);

  detail = gtk_text_new(NULL, NULL);
  vscroll = gtk_vscrollbar_new(GTK_TEXT(detail)->vadj);

  gtk_table_attach(GTK_TABLE(text_table), detail, 0, 1, 0, 1,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, 
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   0, 0);

  gtk_table_attach(GTK_TABLE(text_table), vscroll, 1, 2, 0, 1,
                   GTK_SHRINK, 
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   0, 0);

#ifdef __SIKIGAMI_PDA__
  gtk_widget_set_usize(detail, -1, 32);
#else
  gtk_widget_set_usize(detail, -1, -1);
#endif

  gtk_table_attach(GTK_TABLE(edit_table), text_table, 0, 10, 12, 13,
                   GTK_FILL|GTK_EXPAND|GTK_SHRINK, 
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   0, 0);

  gtk_text_set_editable(GTK_TEXT(detail), TRUE);

  gtk_signal_connect(GTK_OBJECT(detail), "focus_in_event",
		     GTK_SIGNAL_FUNC(set_selection_src_widget), 
		     0);
  gtk_signal_connect(GTK_OBJECT(detail), "focus_out_event",
		     GTK_SIGNAL_FUNC(unset_selection_src_widget), 
		     0);

}

void create_select_dialog(GtkWidget *w, void (*ok_func)(), void (*cancel_func)())
{
  GtkWidget *dialog;
  GtkWidget *y_btn, *n_btn;

  dialog = gtk_dialog_new();

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
		     GTK_SIGNAL_FUNC(close_dialog), NULL);

  gtk_window_set_title(GTK_WINDOW(dialog), NULL);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     w, TRUE, TRUE, 0);

  y_btn = gtk_button_new_with_label(_("OK"));
  n_btn = gtk_button_new_with_label(_("Cancel"));

  gtk_signal_connect(GTK_OBJECT(y_btn), "clicked",
		     GTK_SIGNAL_FUNC(ok_func), dialog);
  gtk_signal_connect(GTK_OBJECT(n_btn), "clicked",
		     GTK_SIGNAL_FUNC(cancel_func), dialog);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     y_btn, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     n_btn, TRUE, TRUE, 0);

  gtk_widget_show(w);
  gtk_widget_show(y_btn);
  gtk_widget_show(n_btn);
  gtk_widget_show(dialog);
  gtk_grab_add(dialog);
}

void check_specified_day(GtkWidget *widget, gpointer data)
{
  int year = 0, month = 0, day = 0;

#ifdef STYLE_COMBO
  year  = gtk_get_entry_num(GTK_COMBO(spec_year)->entry);
  month = gtk_get_entry_num(GTK_COMBO(spec_mon)->entry);
  day   = gtk_get_entry_num(GTK_COMBO(spec_day)->entry);
#else
  year  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spec_year));
  month = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spec_mon));
  day   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spec_day));
#endif

  if(day > get_day_of_month(year, month)){
#ifdef STYLE_COMBO
    set_entry_num(GTK_COMBO(spec_day)->entry, get_day_of_month(year,month));
#else
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spec_day), (gfloat)get_day_of_month(year,month));
#endif
  }
}

void create_specify_combo()
{
  GtkWidget *mesg;
  GtkAdjustment *adjustment;

  spec_table = gtk_table_new(2, 3,  FALSE);

  mesg = gtk_label_new(_("Specify the day\nthat you want to see."));
  gtk_misc_set_padding(GTK_MISC(mesg), 10, 10);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (2001.0 , 2001.0 , 2037.0, 1.0, 25.0, 25.0);
  spec_year = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (1.0 , 1.0 , 12.0, 1.0, 25.0, 25.0);
  spec_mon = gtk_spin_button_new(adjustment, 1.0, 0);

  adjustment = (GtkAdjustment *) gtk_adjustment_new (1.0 , 1.0 , 31.0, 1.0, 25.0, 25.0);
  spec_day = gtk_spin_button_new(adjustment, 1.0, 0);

  gtk_table_attach(GTK_TABLE(spec_table), mesg, 0, 3, 0, 1,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK|GTK_FILL|GTK_EXPAND, 0, 0);
  gtk_table_attach(GTK_TABLE(spec_table), spec_year, 0, 1, 1, 2,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK|GTK_FILL|GTK_EXPAND, 0, 0);
  gtk_table_attach(GTK_TABLE(spec_table), spec_mon,  1, 2, 1, 2,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK|GTK_FILL|GTK_EXPAND, 0, 0);
  gtk_table_attach(GTK_TABLE(spec_table), spec_day,  2, 3, 1, 2,
		   GTK_SHRINK|GTK_FILL|GTK_EXPAND,
		   GTK_SHRINK|GTK_FILL|GTK_EXPAND, 0, 0);

  gtk_signal_connect(GTK_OBJECT(spec_year), "changed",
		     GTK_SIGNAL_FUNC(check_specified_day), (gpointer)NULL);
  gtk_signal_connect(GTK_OBJECT(spec_mon),  "changed",
		     GTK_SIGNAL_FUNC(check_specified_day), (gpointer)NULL);
  gtk_signal_connect(GTK_OBJECT(spec_day),  "changed",
		     GTK_SIGNAL_FUNC(check_specified_day), (gpointer)NULL);
  
  gtk_widget_set_usize(spec_year, 90, -1);
  gtk_widget_set_usize(spec_mon,  60, -1);
  gtk_widget_set_usize(spec_day,  60, -1);

  gtk_widget_show(mesg);
  gtk_widget_show(spec_year);
  gtk_widget_show(spec_mon);
  gtk_widget_show(spec_day);
}

void move_specified_day(GtkWidget *widget, gpointer data)
{
#ifdef STYLE_COMBO
  current_year  = gtk_get_entry_num(GTK_COMBO(spec_year)->entry);
  current_month = gtk_get_entry_num(GTK_COMBO(spec_mon)->entry);
  current_day   = gtk_get_entry_num(GTK_COMBO(spec_day)->entry);
#else
  current_year  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spec_year));
  current_month = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spec_mon));
  current_day   = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spec_day));
#endif
  
  if(sc_current) sched_destroy_list(sc_current);      
  sc_current = get_sched_list(current_year, current_month);
  if(sc_previous) sched_destroy_list(sc_previous);      
  sc_previous = get_sched_previous_list(current_year, current_month);
  
  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);  
  draw_calendar(current_year, current_month, current_day);

  gtk_widget_destroy((GtkWidget *)data);
}

void destroy_specify_date(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy((GtkWidget *)data);
}

void specify_date()
{
  create_specify_combo();
  create_select_dialog(spec_table, move_specified_day, destroy_specify_date);
}

