/* 
   $Id: sgsched_list.c,v 1.6 2001/12/27 08:21:16 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgsched.

   sgsched is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgsched is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgsched; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "config.h"

#include <stdio.h>

#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sgsched.h"

GtkWidget *sched_list;
GtkWidget *sched_vscroll;
GtkObject *sched_adjustment;

static GdkColor glay    = {0, 0x7fff, 0x7fff, 0x7fff}; 
static GdkColor black   = {0,      0,      0,      0}; 
static GdkColor red     = {0, 0xffff,      0,      0}; 
static GdkColor d_red   = {0, 0x7fff,      0,      0}; 
static GdkColor blue    = {0,      0,      0, 0xffff}; 
static GdkColor d_blue  = {0,      0,      0, 0x7fff}; 
static GdkColor green   = {0,      0, 0xffff,      0}; 
static GdkColor d_green = {0,      0, 0x7fff,      0}; 

struct sched_data *sc_all  = NULL;

struct sched_data *sc_current  = NULL;
struct sched_data *sc_previous = NULL;

int current_row = 0;

void select_row(GtkCList *clist, gint row, gint column, GdkEvent *event);
void list_button_press(GtkWidget *list, GdkEventButton *event, gpointer data);
void resize_list(void);
int  comp_time_by_day(time_t t1, time_t t2);
char *get_first_line(char *str);
void set_list(struct sched_data *sched);
void set_lists(struct sched_data *list, time_t start, time_t end);
void set_sched_state(struct sched_data *tmp, int year, int month);

struct sched_data *
get_selected_row_data()
{
  struct sched_data *tmp = NULL;
  tmp = gtk_clist_get_row_data(GTK_CLIST(sched_list), current_row);

  if(tmp){
    if(tmp->orig != NULL) return tmp->orig;
    else                  return tmp;
  }
  
  return NULL;
}

void select_row(GtkCList *clist, gint row, gint column, GdkEvent *event)
{
  current_row = row;
}

void 
list_button_press(GtkWidget *list, GdkEventButton *event, gpointer data)
{
  static unsigned int time = 0;
  struct sched_data *tmp;

  if((event->time - time) < 500){
    tmp = get_selected_row_data();
    if(tmp && (tmp->flag & 0x0f)){
      set_sched_data(tmp);
      set_ok_btn_state(1);
      set_page(1);
    }
  }

  time = event->time;
}

void resize_list()
{
  int w;

  w = gtk_clist_optimal_column_width (GTK_CLIST(sched_list), 0);  
  gtk_clist_set_column_min_width(GTK_CLIST(sched_list), 0, w);  
  gtk_clist_set_column_max_width(GTK_CLIST(sched_list), 0, w);
  gtk_clist_set_column_width(GTK_CLIST(sched_list), 0, w);
}

GtkWidget *
init_list()
{
  GtkWidget *sched_table;
  GdkColormap *colormap;

  char *titles[2];
  int  i;
  char       *home;
  char       dname[BUFSIZ];
  char       fn[BUFSIZ];
  DIR        *dirp;

  sched_table = gtk_table_new(1, 2, FALSE);

  titles[0] = _("Time");
  titles[1] = _("day plan");
  sched_list = gtk_clist_new_with_titles(2, titles);
  gtk_widget_set_usize(sched_list, 1, 1);

  i = gtk_clist_optimal_column_width (GTK_CLIST(sched_list), 0);  
  gtk_clist_set_column_min_width(GTK_CLIST(sched_list), 0, i);  
  gtk_clist_set_column_max_width(GTK_CLIST(sched_list), 0, i);

  gtk_clist_column_titles_passive(GTK_CLIST(sched_list));

  gtk_table_attach(GTK_TABLE(sched_table), sched_list, 0, 1, 0, 1,
                   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);

  gtk_signal_connect (GTK_OBJECT (sched_list), "button_press_event", 
		      GTK_SIGNAL_FUNC (list_button_press), NULL);

  gtk_signal_connect (GTK_OBJECT (sched_list), "select_row", 
		      GTK_SIGNAL_FUNC (select_row), NULL);

  sched_adjustment = gtk_adjustment_new (0.0, 0.0, 1000000.0, 1.0, 25.0, 25.0);
  gtk_clist_set_vadjustment(GTK_CLIST(sched_list), 
			    GTK_ADJUSTMENT(sched_adjustment));

  sched_vscroll = gtk_vscrollbar_new(GTK_ADJUSTMENT(sched_adjustment));
  gtk_table_attach(GTK_TABLE(sched_table), sched_vscroll, 1, 2, 0, 1,
		   GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);

  colormap = gtk_widget_get_colormap(sched_list);

  gdk_color_alloc (colormap, &glay);
  gdk_color_alloc (colormap, &black);

  gdk_color_alloc (colormap, &red);
  gdk_color_alloc (colormap, &blue);
  gdk_color_alloc (colormap, &green);
  gdk_color_alloc (colormap, &d_red);
  gdk_color_alloc (colormap, &d_blue);
  gdk_color_alloc (colormap, &d_green);

  if ((home = getenv("HOME")) != NULL) {
    sprintf(dname, "%s/sgsched",home);
  } else {
    sprintf(dname, "sgsched");
  }

  dirp = opendir(dname);

  if(!dirp) {
    mkdir(dname,0700);
  } else      closedir(dirp);

  sprintf(fn,"%s/schedule", dname);
  sc_all = sched_load_list(fn);

  return sched_table;
}

int comp_time_by_day(time_t t1, time_t t2)
{
  struct tm   *tm;
  struct date a, b;

  tm = localtime(&t1);
  a.year = tm->tm_year + 1900;
  a.mon  = tm->tm_mon  + 1;
  a.mday = tm->tm_mday;

  tm = localtime(&t2);
  b.year = tm->tm_year + 1900;
  b.mon  = tm->tm_mon  + 1;
  b.mday = tm->tm_mday;

  return comp_date(&a, &b);
}

struct sched_data *
get_sched_list(int year, int month)
{
  struct sched_data *tmp = NULL;
  struct sched_data *tail = NULL;
  struct sched_data *h = NULL;
  struct date *date, end;
  int y, m, d;

  y = year, m = month;

  tmp = sched_create_list(y, m, 1, y, m, get_day_of_month(y, m));

  if(!tmp){
    tmp = (struct sched_data *) calloc(1, sizeof(struct sched_data));
  }
  
  tail = tmp;
  while(tail->next){
    tail = tail->next;
  }

  end.year = y;
  end.mon  = m;
  end.mday = get_day_of_month(y, m);

  date = get_next_public_holiday(y, m, 0);

  while(date){
    y = date->year, m = date->mon, d = date->mday;
    if(comp_date(date, &end) > 0) break;

    h = (struct sched_data *) calloc(1, sizeof(struct sched_data));
    
    h->start = get_time_t(y, m, d, 0, 0, 0);
    h->end   = get_time_t(y, m, d, 0, 0, 0);
    h->flag  = HOLIDAY_SCHEDULE;
    h->data  = get_holiday_by_date(y, m, d);
    if(h->data == NULL){
      h->data = _("Substitute Holiday");
    }

    h->prev  = tail;
    tail->next = h;

    tail = tail->next;

    date = get_next_public_holiday(y, m, d);
  }

  y = year, m = month;
  date = get_next_memorialday(y, m, 0);

  while(date){
    y = date->year, m = date->mon, d = date->mday;
    if(comp_date(date, &end) > 0) break;

    h = (struct sched_data *) calloc(1, sizeof(struct sched_data));

    h->start = get_time_t(y, m, d, 0, 0, 0);
    h->end   = get_time_t(y, m, d, 0, 0, 0);
    h->flag  = MEMORIAL_SCHEDULE;
    h->data  = get_memorialday_by_date(y, m, d);

    h->prev  = tail;
    tail->next = h;

    tail = tail->next;

    date = get_next_memorialday(y, m, d);
  }

  tmp = sort_schedule_list(tmp);

  return tmp;
}

struct sched_data *
get_sched_previous_list(int y, int m)
{
  m--;
  if(m < 1) y--, m = 12;
  return get_sched_list(y, m);
}

char *get_first_line(char *str)
{
  int  len;
  char *ret = NULL;
  char *p = NULL, *q = NULL;

  if(str != NULL){
    len = strlen(str);

    if(len > 0){ 
      q = str;
      do{
	p = strchr(q, '\n');
	if(p && ((p - q) > 0)){
	  ret = (char *) calloc(1, (p - q + 1)*sizeof(char));
	  strncpy(ret, q, p - q);
	  break;
	}else if(!p && strlen(q) > 0){
	  ret = strdup(q);
	  break;
	}
	q = p + 1;
      }while(p);
    }
  }

  return ret;
}

void set_list(struct sched_data *sched)
{
  struct tm  tm1, tm2, *tm;  
  char       *s[3];
  char       time[256], p[256];
  int        row;
  int        state;

  tm  = localtime(&sched->start);
  tm1 = *tm;
  tm  = localtime(&sched->end);
  tm2 = *tm;

  state = get_view_state();

  if(!(sched->flag & (HOLIDAY_SCHEDULE|MEMORIAL_SCHEDULE))
       && !(sched->flag & DAYOFF_SCHEDULE)){
    switch(state){
    case DAY_VIEW:
      if(sched->start == sched->end){
	sprintf(time, "%02d:%02d", tm1.tm_hour, tm1.tm_min);
      }else if(sched->end - sched->start < 86400){
	sprintf(p, "%02d:%02d", tm1.tm_hour, tm1.tm_min); 
	sprintf(time, "%s - %02d:%02d", p, tm2.tm_hour, tm2.tm_min);
      }else{
	sprintf(p, "%2d/%2d", tm1.tm_mon + 1, tm1.tm_mday); 
	sprintf(time, "%s - %2d/%2d", p, tm2.tm_mon + 1, tm2.tm_mday);
      }
      break;
      
    case WEEK_VIEW:
      case MONTH_VIEW:
	if(sched->end - sched->start < 86400){
	  sprintf(time, "%2d/%2d", tm1.tm_mon + 1, tm1.tm_mday);
	}else{
	  sprintf(p, "%2d/%2d", tm1.tm_mon + 1, tm1.tm_mday); 
	  sprintf(time, "%s - %2d/%2d", p, tm2.tm_mon + 1, tm2.tm_mday);
	}
	break;
    default:break;
    }
  }else{
    sprintf(time, "%2d/%2d", tm1.tm_mon + 1, tm1.tm_mday);
  }
    
  s[0] = time;
  s[1] = get_first_line(sched->data);
  s[2] = NULL;
    
  row = gtk_clist_append(GTK_CLIST(sched_list), s);
  gtk_clist_set_row_data(GTK_CLIST(sched_list), row, sched);
  
  switch(sched->flag & ~DAYOFF_SCHEDULE){
  case NORMAL_SCHEDULE:
    gtk_clist_set_foreground(GTK_CLIST(sched_list), row, &black);
    break;
    
  case WEEKLY_SCHEDULE:
    gtk_clist_set_foreground(GTK_CLIST(sched_list), row, &d_green);
    break;
    
  case MONTHLY_SCHEDULE:
    gtk_clist_set_foreground(GTK_CLIST(sched_list), row, &d_blue);
    break;
    
  case HOLIDAY_SCHEDULE:
    gtk_clist_set_foreground(GTK_CLIST(sched_list), row, &red);
    break;
    default:break;
  }

}

int  c_state[32];

void set_lists(struct sched_data *list, time_t start, time_t end)
{
  struct sched_data *tmp;

  if(list){
    tmp = list->next;
    
    while(tmp){
      if(!(tmp->end < start || tmp->start > end))
	set_list(tmp);
      tmp = tmp->next;
    }
  }
}

void sched_set_list(int year, int month, int day)
{
  int        w;
  time_t     t1 = 0, t2 = 0;

  gtk_clist_clear(GTK_CLIST(sched_list));

  switch(get_view_state()){
  case DAY_VIEW:
    t1 = get_time_t(year, month,day,  0,  0,  0);
    t2 = get_time_t(year, month,day, 23, 59, 59);
    break;

  case WEEK_VIEW:
    w = get_dayweek(year, month, day);
    t1 = get_time_t(year, month, day, 0, 0, 0)    - (w * 86400);
    t2 = get_time_t(year, month, day, 23, 59, 59) + ((6 - w) * 86400);
    break;

  case MONTH_VIEW:
    t1 = get_time_t(year, month, 1, 0, 0, 0);
    t2 = get_time_t(year, month, get_day_of_month(year, month) , 23, 59, 59);
    break;
  default:break;
  }

  if(!sc_current)  sc_current  = get_sched_list(year, month);
  if(!sc_previous) sc_previous = get_sched_previous_list(year, month);
  
  set_lists(sc_previous, t1, t2);
  set_lists(sc_current,  t1, t2);

  resize_list();
}

void set_sched_state(struct sched_data *tmp, int year, int month)
{
  struct tm *tm;
  time_t t1, t2, a;

  t1 = get_time_t(year, month, 1, 0, 0, 0);
  t2 = get_time_t(year, month, get_day_of_month(year, month) , 23, 59, 59);

  while(tmp){
    if(!(tmp->flag & (HOLIDAY_SCHEDULE|MEMORIAL_SCHEDULE))){
      if(!(tmp->end < t1 || tmp->start > t2)){
	tm = localtime(&tmp->start);
	a = get_time_t(tm->tm_year + 1900, tm->tm_mon + 1,tm->tm_mday,
		       is_pm(tmp->start) ? 12 : 0, 0, 0);
	
	if(tmp->start != tmp->end){
	  while(a <= tmp->end){
	    tm = localtime(&a);
	    
	    if((tm->tm_mon + 1) == month){
	      if(tmp->flag & DAYOFF_SCHEDULE)
		c_state[tm->tm_mday] |= (1 << 3);
	      else	    
		c_state[tm->tm_mday] |= (!is_pm(a)) | (is_pm(a) << 1);
	    }
	    
	    a += 43200;
	  }
	}else{
	  if(tmp->flag & DAYOFF_SCHEDULE)
	    c_state[tm->tm_mday] |= (1 << 3);
	  else	    
	    c_state[tm->tm_mday] |= (!is_pm(a)) | (is_pm(a) << 1);
	}
      }
    }
    tmp = tmp->next;
  }
}
  
void set_calendar_state(int year, int month)
{
  bzero(c_state, sizeof(int)*32);

  if(!sc_current)  sc_current  = get_sched_list(year, month);
  if(!sc_previous) sc_previous = get_sched_previous_list(year, month);

  if(sc_previous)  set_sched_state(sc_previous->next, year, month);
  if(sc_current)   set_sched_state(sc_current->next, year, month);
}
