/* 
   $Id: sgsched_main.c,v 1.8 2001/12/27 08:21:16 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgsched.

   sgsched is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgsched is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgsched; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "sgsched.h"

GSList    *view_group = NULL;

static GnomeUIInfo file_menu[] = {
  {
    GNOME_APP_UI_ITEM,
    N_("_New"), NULL,
    GTK_SIGNAL_FUNC(new_press), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM,
    N_("_Quit"), NULL,
    GTK_SIGNAL_FUNC(quit_ev), NULL, NULL,
    GNOME_APP_PIXMAP_STOCK , GNOME_STOCK_MENU_QUIT,
    0, GDK_CONTROL_MASK, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu[] = {
  {
    GNOME_APP_UI_ITEM,
    N_("Modify"), NULL,
    GTK_SIGNAL_FUNC(edit_press), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM,
    N_("_Delete"), NULL,
    GTK_SIGNAL_FUNC(del_press), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },

  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_CUT_ITEM((gpointer)cut_command, NULL),
  GNOMEUIINFO_MENU_COPY_ITEM((gpointer)copy_command, NULL),
  GNOMEUIINFO_MENU_PASTE_ITEM((gpointer)paste_command, NULL),
  GNOMEUIINFO_MENU_SELECT_ALL_ITEM((gpointer)selall_text, NULL),

  GNOMEUIINFO_END
};

static GnomeUIInfo move_menu[] = {
  {
    GNOME_APP_UI_ITEM,
    N_("Go to Today"), NULL,
    GTK_SIGNAL_FUNC(move_today), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },

  {
    GNOME_APP_UI_ITEM,
    N_("Go to Specified Day"), NULL,
    GTK_SIGNAL_FUNC(specify_date), NULL, NULL,
    GNOME_APP_PIXMAP_NONE , NULL,
    0, GDK_CONTROL_MASK, NULL
  },

  GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
  GNOMEUIINFO_SUBTREE(N_("_File"),file_menu),
  GNOMEUIINFO_SUBTREE(N_("_Edit"),edit_menu),
  GNOMEUIINFO_SUBTREE(N_("_Move"),move_menu),
  GNOMEUIINFO_END
};

GtkWidget *notebook;
GtkWidget *calendar;
GtkWidget *sched_table;
GtkWidget *sched_edit;
GtkWidget *selection_src;

int    current_year;
int    current_month;
int    current_day;
int    current_week;

extern struct sched_data *sc_current;
extern struct sched_data *sc_previous;

void set_selection_src_widget(GtkWidget *widget, GdkEventFocus *ev)
{
  selection_src = widget;
}

void unset_selection_src_widget(GtkWidget *widget, GdkEventFocus *ev)
{
  selection_src = NULL;
}

void set_page(int i)
{
  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), i);
}

void move_today(GtkWidget *widget, gpointer data)
{
  time_t    t;
  struct tm *tm;

  time(&t);
  tm = localtime(&t);
  current_year  = tm->tm_year + 1900; 
  current_month = tm->tm_mon + 1;
  current_day   = tm->tm_mday;
  current_week  = (tm->tm_mday + get_dayweek(current_year, current_month, 1)) / 7;

  if(sc_current) sched_destroy_list(sc_current);      
  sc_current = get_sched_list(current_year, current_month);
  if(sc_previous) sched_destroy_list(sc_previous);      
  sc_previous = get_sched_previous_list(current_year, current_month);
  
  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);  
  draw_calendar(current_year, current_month, current_day);
}

void close_dialog(GtkWidget *widget, gpointer data)
{
  gtk_grab_remove(widget);
}

void message_btn(char *message, void (*y_func)(), void (*n_func)())
{
  GtkWidget *dialog;
  GtkWidget *d_mesg;
  GtkWidget *y_btn, *n_btn;

  dialog = gtk_dialog_new();

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
		     GTK_SIGNAL_FUNC(close_dialog), NULL);

  gtk_window_set_title(GTK_WINDOW(dialog), NULL);

  d_mesg = gtk_label_new(message);

  gtk_misc_set_padding(GTK_MISC(d_mesg), 10, 10);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     d_mesg, TRUE, TRUE, 0);

  y_btn = gtk_button_new_with_label(_("Yes"));
  n_btn = gtk_button_new_with_label(_("No"));

  gtk_signal_connect(GTK_OBJECT(y_btn), "clicked",
		     GTK_SIGNAL_FUNC(y_func), dialog);
  gtk_signal_connect(GTK_OBJECT(n_btn), "clicked",
		     GTK_SIGNAL_FUNC(n_func), dialog);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     y_btn, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     n_btn, TRUE, TRUE, 0);

  gtk_widget_show(d_mesg);
  gtk_widget_show(y_btn);
  gtk_widget_show(n_btn);
  gtk_widget_show(dialog);
  gtk_grab_add(dialog);
}

void quit_with_save()
{
  ok_press(NULL, NULL);
  quit();
}

void quit()
{
  char       *home;
  char       dname[BUFSIZ];
  char       fn[BUFSIZ];

  if ((home = getenv("HOME")) != NULL) {
    sprintf(dname, "%s/sgsched",home);
  } else {
    sprintf(dname, "sgsched");
  }
  
  sprintf(fn,"%s/schedule", dname);
  sched_save_list(fn);

  gtk_main_quit();
}

void quit_ev(GtkWidget *widget, gpointer data)
{
  int num;
  num = (int)gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  if(num == 1 && get_ok_btn_state() != 2)
    message_btn(
		_("Do you want to save the schedule\nthat you are editting now ?"),
		quit_with_save, quit);
  else
    quit();
}

void init_view(GtkWidget *app)
{
  GtkWidget *table;

  table = gtk_table_new(3, 2, FALSE);
  gtk_notebook_append_page(GTK_NOTEBOOK(app), table, NULL);

  /* create calendar */
  calendar = init_calendar();
#ifdef __SIKIGAMI_PDA__
  gtk_table_attach(GTK_TABLE(table), calendar, 0, 2, 1, 2,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND, 
		   GTK_SHRINK, 0, 0);
#else
  gtk_table_attach(GTK_TABLE(table), calendar, 0, 1, 1, 3,
                   GTK_SHRINK|GTK_FILL|GTK_EXPAND, 
		   GTK_SHRINK|GTK_FILL, 0, 0);
#endif

  /* create schedule list */
  sched_table = init_list();
#ifdef __SIKIGAMI_PDA__
  gtk_table_attach(GTK_TABLE(table), sched_table, 0, 2, 2, 3,
#else
  gtk_table_attach(GTK_TABLE(table), sched_table, 1, 2, 1, 3,
#endif
                   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   0, 0);

}

void copy_command()
{
  int i;
  
  i = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  if(i == 1) copy_text();
  else       copy_sched();
}

void copy_text()
{
  if(selection_src != NULL)
    gtk_editable_copy_clipboard(GTK_EDITABLE(selection_src));
}

struct sched_data *copy;

void copy_sched()
{
  struct sched_data *tmp;

  tmp = get_selected_row_data();
  if(tmp == NULL) return;
  if(!(tmp->flag & 0xf)) return;

  if(copy != NULL){
    if(copy->location) free(copy->location);
    if(copy->person)   free(copy->person);
    if(copy->data)     free(copy->data);
    free(copy);
  }

  copy = calloc(1, sizeof(struct sched_data));
  *copy = *tmp;
  copy->prev = NULL;
  copy->next = NULL;
  if(copy->location) copy->location = strdup(tmp->location);
  if(copy->person)   copy->person   = strdup(tmp->person);
  if(copy->data)     copy->data     = strdup(tmp->data);
}

void cut_command()
{
  int i;

  i = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  if(i == 1) cut_text();
  else       cut_sched();
}

void cut_text()
{
  if(selection_src != NULL)
    gtk_editable_cut_clipboard(GTK_EDITABLE(selection_src));
}

void cut_sched()
{
  struct sched_data *tmp;

  copy_sched();

  tmp = get_selected_row_data();
  sched_del_data(tmp);

  sched_destroy_list(sc_current);      
  sc_current = get_sched_list(current_year, current_month);

  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);
  draw_calendar(current_year, current_month, current_day);
}

void paste_command()
{
  int i;

  i = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  if(i == 1) paste_text();
  else       paste_sched();
}

void paste_text()
{
  if(selection_src != NULL)
    gtk_editable_paste_clipboard(GTK_EDITABLE(selection_src));
}

void paste_sched()
{
  struct tm *tm = NULL;
  struct sched_data *tmp;

  if(get_view_state() != DAY_VIEW) return;
  if(copy == NULL) return;

  tmp = calloc(1, sizeof(struct sched_data));
  *tmp = *copy;
  tmp->prev = NULL;
  tmp->next = NULL;
  if(copy->location) tmp->location = strdup(copy->location);
  if(copy->person)   tmp->person   = strdup(copy->person);
  if(copy->data)     tmp->data     = strdup(copy->data);

  tm = localtime(&tmp->start);
  if(tm)
    tmp->start = get_time_t(current_year, current_month, current_day,
			     tm->tm_hour, tm->tm_min, 0);

  tmp->end = tmp->start + copy->end - copy->start;

  sched_add_data(tmp);

  sched_destroy_list(sc_current);      
  sc_current = get_sched_list(current_year, current_month);

  sched_set_list(current_year, current_month, current_day);
  set_calendar_state(current_year, current_month);
  draw_calendar(current_year, current_month, current_day);
}


void selall_text()
{
  if(selection_src != NULL)
    gtk_editable_select_region(GTK_EDITABLE(selection_src), 0,
			       gtk_text_get_length(GTK_TEXT(selection_src)));
}

int main(int argc, char **argv)
{
  GtkWidget *app;
  int status;

  status = SGCheckDinvoke(argv[0]);
  if(status > 0) return -1;

  bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain(PACKAGE);

  gtk_set_locale();

  gnome_init (PACKAGE, VERSION, argc, argv);
  app = gnome_app_new(PACKAGE, _("sgsched"));
  gtk_widget_set_name(app, PACKAGE);

  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
		      GTK_SIGNAL_FUNC (quit_ev), NULL);
  
  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);

  gnome_app_set_contents(GNOME_APP(app), notebook);

  init_view(notebook);
  init_edit(notebook);

  gnome_app_create_menus(GNOME_APP(app), main_menu);

  gtk_widget_show_all(app);

  init_holiday();

  move_today(NULL, NULL);

  gtk_main();

  return 1;
}
